@@ -41,12 +41,7 @@
 #include "gmm/gmm.h"
 #include "gmm/gmm_inoutput.h"
 
-/* some Getfem++ types that we will be using */
-using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
-using bgeot::base_node;  /* geometrical nodes(derived from base_small_vector)*/
-using bgeot::scalar_type; /* = double */
-using bgeot::size_type;   /* = unsigned long */
-using bgeot::base_matrix; /* small dense matrix. */
+#include "crack_exact_solution.h"
 
 /* definition of some matrix/vector types. These ones are built
  * using the predefined types in Gmm++
@@ -55,255 +50,6 @@ typedef getfem::modeling_standard_sparse_vector sparse_vector;
 typedef getfem::modeling_standard_sparse_matrix sparse_matrix;
 typedef getfem::modeling_standard_plain_vector  plain_vector;
 
-/**************************************************************************/
-/*  Exact solution.                                                       */
-/**************************************************************************/
-
-#define VALIDATE_XFEM
-
-#ifdef VALIDATE_XFEM
-
-
-
-/* returns sin(theta/2) where theta is the angle
-   of 0-(x,y) with the axis Ox */
-scalar_type sint2(scalar_type x, scalar_type y) {
-  scalar_type r = sqrt(x*x+y*y);
-  if (r == 0) return 0;
-  else return (y<0 ? -1:1) * sqrt(gmm::abs(r-x)/(2*r));
-  // sometimes (gcc3.3.2 -O3), r-x < 0 ....
-}
-scalar_type cost2(scalar_type x, scalar_type y) {
-  scalar_type r = sqrt(x*x+y*y);
-  if (r == 0) return 0;
-  else return sqrt(gmm::abs(r+x)/(2*r));
-}
-/* analytical solution for a semi-infinite crack [-inf,a] in an
-   infinite plane submitted to +sigma above the crack
-   and -sigma under the crack. (The crack is directed along the x axis).
-   
-   nu and E are the poisson ratio and young modulus
-   
-   solution taken from "an extended finite elt method with high order
-   elts for curved cracks", Stazi, Budyn,Chessa, Belytschko
-*/
-
-void elasticite2lame(const scalar_type young_modulus,
-		     const scalar_type poisson_ratio, 
-		     scalar_type& lambda, scalar_type& mu) {
-  mu = young_modulus/(2*(1+poisson_ratio));
-  lambda = 2*mu*poisson_ratio/(1-poisson_ratio);
-}
-
-void sol_ref_infinite_plane(scalar_type nu, scalar_type E, scalar_type sigma,
-			    scalar_type a, scalar_type xx, scalar_type y,
-			    base_small_vector& U, int mode,
-			    base_matrix *pgrad) {
-  scalar_type x  = xx-a; /* the eq are given relatively to the crack tip */
-  //scalar_type KI = sigma*sqrt(M_PI*a);
-  scalar_type r = std::max(sqrt(x*x+y*y),1e-16);
-  scalar_type sqrtr = sqrt(r), sqrtr3 = sqrtr*sqrtr*sqrtr;
-  scalar_type cost = x/r, sint = y/r;
-  scalar_type theta = atan2(y,x);
-  scalar_type s2 = sin(theta/2); //sint2(x,y);
-  scalar_type c2 = cos(theta/2); //cost2(x,y);
-  // scalar_type c3 = cos(3*theta/2); //4*c2*c2*c2-3*c2; /* cos(3*theta/2) */
-  // scalar_type s3 = sin(3*theta/2); //4*s2*c2*c2-s2;  /* sin(3*theta/2) */
-
-  scalar_type lambda, mu;
-  elasticite2lame(E,nu,lambda,mu);
-
-  U.resize(2);
-  if (pgrad) (*pgrad).resize(2,2);
-  scalar_type C= 1./E * (mode == 1 ? 1. : (1+nu));
-  if (mode == 1) {
-    scalar_type A=2+2*mu/(lambda+2*mu);
-    scalar_type B=-2*(lambda+mu)/(lambda+2*mu);
-    U[0] = sqrtr/sqrt(2*M_PI) * C * c2 * (A + B*cost);
-    U[1] = sqrtr/sqrt(2*M_PI) * C * s2 * (A + B*cost);
-    if (pgrad) {
-      (*pgrad)(0,0) = C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (cost*c2*A-cost*cost*c2*B+sint*s2*A+sint*s2*B*cost+2*c2*B);
-      (*pgrad)(1,0) = -C/(2*sqrt(2*M_PI)*sqrtr)
-	* (-sint*c2*A+sint*c2*B*cost+cost*s2*A+cost*cost*s2*B);
-      (*pgrad)(0,1) = C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (cost*s2*A-cost*cost*s2*B-sint*c2*A-sint*c2*B*cost+2*s2*B);
-      (*pgrad)(1,1) = C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (sint*s2*A-sint*s2*B*cost+cost*c2*A+cost*cost*c2*B);
-    }
-  } else if (mode == 2) {
-    scalar_type C1 = (lambda+3*mu)/(lambda+mu);
-    U[0] = sqrtr/sqrt(2*M_PI) * C * s2 * (C1 + 2 + cost);
-    U[1] = sqrtr/sqrt(2*M_PI) * C * c2 * (C1 - 2 + cost) * (-1.);
-    if (pgrad) {
-      (*pgrad)(0,0) = C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (cost*s2*C1+2*cost*s2-cost*cost*s2-sint*c2*C1
-	   -2*sint*c2-sint*cost*c2+2*s2);
-      (*pgrad)(1,0) = C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (sint*s2*C1+2*sint*s2-sint*s2*cost+cost*c2*C1
-	   +2*cost*c2+cost*cost*c2);
-      (*pgrad)(0,1) = -C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (cost*c2*C1-2*cost*c2-cost*cost*c2+sint*s2*C1
-	   -2*sint*s2+sint*s2*cost+2*c2);
-      (*pgrad)(1,1) =  C/(2.*sqrt(2*M_PI)*sqrtr)
-	* (-sint*c2*C1+2*sint*c2+sint*cost*c2+cost*s2*C1
-	   -2*cost*s2+cost*cost*s2);
-    }
-  } else if (mode == 100) {
-    U[0] = - sqrtr3 * (c2 + 4./3 *(7*mu+3*lambda)/(lambda+mu)*c2*s2*s2
-		       -1./3*(7*mu+3*lambda)/(lambda+mu)*c2);
-    U[1] = - sqrtr3 * (s2+4./3*(lambda+5*mu)/(lambda+mu)*s2*s2*s2
-		       -(lambda+5*mu)/(lambda+mu)*s2);
-    if (pgrad) {
-      (*pgrad)(0,0) = 2*sqrtr*(-6*cost*c2*mu+7*cost*c2*c2*c2*mu
-			       -3*cost*c2*lambda+3*cost*c2*c2*c2*lambda
-			       -2*sint*s2*mu
-			       +7*sint*s2*c2*c2*mu-sint*s2*lambda
-			       +3*sint*s2*c2*c2*lambda)/(lambda+mu);
-      (*pgrad)(1,0) = -2*sqrtr*(6*sint*c2*mu-7*sint*c2*c2*c2*mu
-				+3*sint*c2*lambda-3*sint*c2*c2*c2*lambda
-				-2*cost*s2*mu
-				+7*cost*s2*c2*c2*mu-cost*s2*lambda
-				+3*cost*s2*c2*c2*lambda)/(lambda+mu);
-      (*pgrad)(0,1) = 2*sqrtr*(-2*cost*s2*mu-cost*s2*lambda
-			       +cost*s2*c2*c2*lambda+5*cost*s2*c2*c2*mu
-			       +4*sint*c2*mu
-			       +sint*c2*lambda-sint*c2*c2*c2*lambda
-			       -5*sint*c2*c2*c2*mu)/(lambda+mu);
-      (*pgrad)(1,1) = 2*sqrtr*(-2*sint*s2*mu-sint*s2*lambda
-			       +sint*s2*c2*c2*lambda+5*sint*s2*c2*c2*mu
-			       -4*cost*c2*mu
-			       -cost*c2*lambda+cost*c2*c2*c2*lambda
-			       +5*cost*c2*c2*c2*mu)/(lambda+mu);
-    }
-  } else if (mode == 101) {
-    U[0] = -4*sqrtr3*s2*(-lambda-2*mu+7*lambda*c2*c2
-			 +11*mu*c2*c2)/(3*lambda-mu);
-    U[1] = -4*sqrtr3*c2*(-3*lambda+3*lambda*c2*c2-mu*c2*c2)/(3*lambda-mu);
-    if (pgrad) {
-      (*pgrad)(0,0) = -6*sqrtr*(-cost*s2*lambda-2*cost*s2*mu
-				+7*cost*s2*lambda*c2*c2
-				+11*cost*s2*mu*c2*c2+5*sint*c2*lambda
-				+8*sint*c2*mu-7*sint*c2*c2*c2*lambda
-				-11*sint*c2*c2*c2*mu)/(3*lambda-mu);
-      (*pgrad)(1,0) = -6*sqrtr*(-sint*s2*lambda-2*sint*s2*mu
-				+7*sint*s2*lambda*c2*c2
-				+11*sint*s2*mu*c2*c2-5*cost*c2*lambda
-				-8*cost*c2*mu+7*cost*c2*c2*c2*lambda
-				+11*cost*c2*c2*c2*mu)/(3*lambda-mu);
-      (*pgrad)(0,1) = -6*sqrtr*(-3*cost*c2*lambda+3*cost*c2*c2*c2*lambda
-				-cost*c2*c2*c2*mu-sint*s2*lambda
-				+3*sint*s2*lambda*c2*c2
-				-sint*s2*mu*c2*c2)/(3*lambda-mu);
-      (*pgrad)(1,1) = 6*sqrtr*(3*sint*c2*lambda
-			       -3*sint*c2*c2*c2*lambda+sint*c2*c2*c2*mu
-			       -cost*s2*lambda+3*cost*s2*lambda*c2*c2
-			       -cost*s2*mu*c2*c2)/(3*lambda-mu);
-    }
-
-  } else if (mode == 10166666) {
-
-    U[0] = 4*sqrtr3*s2*(-lambda+lambda*c2*c2-3*mu*c2*c2)/(lambda-3*mu);
-    U[1] = 4*sqrtr3*c2*(-3*lambda-6*mu+5*lambda*c2*c2+9*mu*c2*c2)/(lambda-3*mu);
-    if (pgrad) {
-      (*pgrad)(0,0) = 6*sqrtr*(-cost*s2*lambda+cost*s2*lambda*c2*c2-
-			       3*cost*s2*mu*c2*c2-2*sint*c2*mu+sint*c2*lambda-
-			       sint*c2*c2*c2*lambda
-			       +3*sint*c2*c2*c2*mu)/(lambda-3*mu);
-      (*pgrad)(1,0) = 6*sqrtr*(-sint*s2*lambda+sint*s2*lambda*c2*c2-
-			       3*sint*s2*mu*c2*c2+2*cost*c2*mu-cost*c2*lambda+
-			       cost*c2*c2*c2*lambda
-			       -3*cost*c2*c2*c2*mu)/(lambda-3*mu);
-      (*pgrad)(0,1) = 6*sqrtr*(-3*cost*c2*lambda-6*cost*c2*mu
-			       +5*cost*c2*c2*c2*lambda+
-			       9*cost*c2*c2*c2*mu-sint*s2*lambda-2*sint*s2*mu+
-			       5*sint*s2*lambda*c2*c2
-			       +9*sint*s2*mu*c2*c2)/(lambda-3*mu);
-      (*pgrad)(1,1) = -6*sqrtr*(3*sint*c2*lambda+6*sint*c2*mu
-				-5*sint*c2*c2*c2*lambda-
-				9*sint*c2*c2*c2*mu-cost*s2*lambda-2*cost*s2*mu+
-				5*cost*s2*lambda*c2*c2
-				+9*cost*s2*mu*c2*c2)/(lambda-3*mu);
-    }
-  } else assert(0);
-  if (isnan(U[0]))
-    cerr << "raaah not a number ... nu=" << nu << ", E=" << E << ", sig="
-	 << sigma << ", a=" << a << ", xx=" << xx << ", y=" << y << ", r="
-	 << r << ", sqrtr=" << sqrtr << ", cost=" << cost << ", U=" << U[0]
-	 << "," << U[1] << endl;
-  assert(!isnan(U[0]));
-  assert(!isnan(U[1]));
-}
-
-struct exact_solution {
-  getfem::mesh_fem_global_function mf;
-  getfem::base_vector U;
-
-  exact_solution(getfem::mesh &me) : mf(me) {}
-  
-  void init(int mode, scalar_type lambda, scalar_type mu,
-	    getfem::level_set &ls) {
-    std::vector<getfem::pglobal_function> cfun(4);
-    for (size_type i = 0; i < 4; ++i) {
-      /* use the singularity */
-      getfem::abstract_xy_function *s = 
-	new getfem::crack_singular_xy_function(i);
-      cfun[i] = getfem::global_function_on_level_set(ls, *s);
-    }
-
-    mf.set_functions(cfun);
-    
-    mf.set_qdim(1);
-    
-    U.resize(8); assert(mf.nb_dof() == 4);
-    getfem::base_vector::iterator it = U.begin();
-    scalar_type coeff=0.;
-    switch(mode) {
-      case 1: {
-	scalar_type A=2+2*mu/(lambda+2*mu), B=-2*(lambda+mu)/(lambda+2*mu);
-	/* "colonne" 1: ux, colonne 2: uy */
-	*it++ = 0;       *it++ = A-B; /* sin(theta/2) */
-	*it++ = A+B;     *it++ = 0;   /* cos(theta/2) */
-	*it++ = -B;      *it++ = 0;   /* sin(theta/2)*sin(theta) */ 
-	*it++ = 0;       *it++ = B;   /* cos(theta/2)*cos(theta) */
-	coeff = 1/sqrt(2*M_PI);
-      } break;
-      case 2: {
-	scalar_type C1 = (lambda+3*mu)/(lambda+mu); 
-	*it++ = C1+2-1;   *it++ = 0;
-	*it++ = 0;      *it++ = -(C1-2+1);
-	*it++ = 0;      *it++ = 1;
-	*it++ = 1;      *it++ = 0;
-	coeff = 2*(mu+lambda)/(lambda+2*mu)/sqrt(2*M_PI);
-      } break;
-      default:
-	assert(0);
-	break;
-    }
-    gmm::scale(U, coeff);
-  }
-};
-
-base_small_vector sol_f(const base_node &x) {
-  int N = x.size();
-  base_small_vector res(N);
-  return res;
-}
-
-#else
-
-base_small_vector sol_f(const base_node &x) {
-  int N = x.size();
-  base_small_vector res(N); res[N-1] = x[N-1];
-  return res;
-}
-
-#endif
-
-
-
-
-
 /**************************************************************************/
 /*  Structure for the crack problem.                                      */
 /**************************************************************************/
@@ -321,7 +67,22 @@ struct crack_problem {
   getfem::mesh_fem mf_partition_of_unity;
   getfem::mesh_fem_product mf_product;
   getfem::mesh_fem_sum mf_u_sum;
-  getfem::spider_fem *spider;
+  
+  base_small_vector translation;
+
+  struct spider_param {
+    getfem::spider_fem *fem;
+    scalar_type theta0;
+    scalar_type radius;
+    unsigned Nr;
+    unsigned Ntheta;
+    int K;
+    int bimat_enrichment;
+    scalar_type epsilon;
+  };
+
+  spider_param spider;
+
   getfem::mesh_fem mf_us;
 
   getfem::mesh_fem& mf_u() { return mf_u_sum; }
@@ -331,7 +92,7 @@ struct crack_problem {
   getfem::mesh_fem mf_rhs;   /* mesh_fem for the right hand side (f(x),..)   */
   getfem::mesh_fem mf_p;     /* mesh_fem for the pressure for mixed form     */
 #ifdef VALIDATE_XFEM
-  exact_solution exact_sol;
+  crack_exact_solution exact_sol;
 #endif
   
   
@@ -339,12 +100,7 @@ struct crack_problem {
   double lambda_up, lambda_down, mu_up, mu_down;  /*Lame coeff for bimaterial case*/
   getfem::level_set ls;      /* The two level sets defining the crack.       */
   getfem::level_set ls2, ls3; /* The two level-sets defining the add. cracks.*/
-  base_small_vector translation;
-  scalar_type theta0;
-  scalar_type spider_radius;
-  unsigned spider_Nr;
-  unsigned spider_Ntheta;
-  int spider_K;
+ 
   scalar_type residual;      /* max residual for the iterative solvers      */
   bool mixed_pressure, add_crack;
   unsigned dir_with_mult;
@@ -441,43 +197,8 @@ void crack_problem::init(void) {
   cout << "FEM_TYPE="  << FEM_TYPE << "\n";
   cout << "INTEGRATION=" << INTEGRATION << "\n";
 
-  spider_radius = PARAM.real_value("SPIDER_RADIUS","spider_radius");
-  spider_Nr = PARAM.int_value("SPIDER_NR","Spider_Nr ");
-  spider_Ntheta = PARAM.int_value("SPIDER_NTHETA","Ntheta ");
-  spider_K = PARAM.int_value("SPIDER_K","K ");
-
-  translation.resize(2); 
-  translation[0] =0.5;
-  translation[1] =0.;
-  theta0 =0;
-  
-  /* First step : build the mesh */
-  bgeot::pgeometric_trans pgt = 
-    bgeot::geometric_trans_descriptor(MESH_TYPE);
-  size_type N = pgt->dim();
-  std::vector<size_type> nsubdiv(N);
-  std::fill(nsubdiv.begin(),nsubdiv.end(),
-	    PARAM.int_value("NX", "Nomber of space steps "));
-  getfem::regular_unit_mesh(mesh, nsubdiv, pgt,
-			    PARAM.int_value("MESH_NOISED") != 0);
-  base_small_vector tt(N); tt[1] = -0.5;
-  mesh.translation(tt); 
-  
-  datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
-  residual = PARAM.real_value("RESIDUAL");
-  if (residual == 0.) residual = 1e-10;
-  enr_area_radius = PARAM.real_value("RADIUS_ENR_AREA",
-				     "radius of the enrichment area");
-  
   bimaterial = PARAM.int_value("BIMATERIAL", "bimaterial interface crack");
-
-  GLOBAL_FUNCTION_MF = PARAM.string_value("GLOBAL_FUNCTION_MF");
-  GLOBAL_FUNCTION_U = PARAM.string_value("GLOBAL_FUNCTION_U");
-
-  reference_test = PARAM.int_value("REFERENCE_TEST", "Reference test"); 
   
-
-
   if (bimaterial == 1){
     mu = PARAM.real_value("MU", "Lame coefficient mu"); 
     mu_up = PARAM.real_value("MU_UP", "Lame coefficient mu"); 
@@ -492,6 +213,95 @@ void crack_problem::init(void) {
     lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
   }
   
+  
+  spider.radius = PARAM.real_value("SPIDER_RADIUS","spider_radius");
+  spider.Nr = PARAM.int_value("SPIDER_NR","Spider_Nr ");
+  spider.Ntheta = PARAM.int_value("SPIDER_NTHETA","Ntheta ");
+  spider.K = PARAM.int_value("SPIDER_K","K ");
+  spider.theta0 = 0;
+  spider.bimat_enrichment = PARAM.int_value("SPIDER_BIMAT_ENRICHMENT","spider_bimat_enrichment");
+
+  /* The following constants are taken from the Chang and Xu paper of 2007 
+     titled The singular stress field and stress intensity factors of a crack 
+     terminating at a bimaterial interface published in IJMECSCI. epsilon 
+     being the constant present in the asymptotic displacement analytic solution: 
+     r^{1/2} * cos (\epsilon Ln(r)) * f(\theta) and  r^{1/2} * sin (\epsilon Ln(r)) * f(\theta).
+  */
+  
+  if (spider.bimat_enrichment == 1){
+    scalar_type nu1 = (lambda_up) / (2.*lambda_up + mu_up);
+    scalar_type nu2 = (lambda_down) / (2.*lambda_down + mu_down);
+    scalar_type kappa1 = 3. - 4. * nu1;
+    scalar_type kappa2 = 3. - 4. * nu2;
+    if (lambda_up == lambda_down && mu_up == mu_down)
+      cout << "ERROR... Connot use the spider bimaterial enrichment with an isotropic homogenuous material (beta = 0/0!!!)... You should either use a bimaterial or disable the spider bimaterial enrichment" << endl;
+    
+    scalar_type beta = (mu_up*(kappa2-1.) - mu_down*(kappa1-1.)) / (mu_up*(kappa2+1.) - mu_down*(kappa1+1.));
+      spider.epsilon = 1./(2.*M_PI) * log( (1.-beta) / (1.+beta) );
+  
+    
+    //spider.epsilon = PARAM.real_value("SPIDER_BIMAT_ENRICHMENT","spider_bimat_enrichment");
+  }
+  
+  translation.resize(2); 
+  translation[0] =0.5;
+  translation[1] =0.;
+
+  /* First step : build the mesh */
+  bgeot::pgeometric_trans pgt = 
+    bgeot::geometric_trans_descriptor(MESH_TYPE);
+  size_type N = pgt->dim();
+  std::vector<size_type> nsubdiv(N);
+  std::fill(nsubdiv.begin(),nsubdiv.end(),
+	    PARAM.int_value("NX", "Nomber of space steps "));
+  getfem::regular_unit_mesh(mesh, nsubdiv, pgt,
+			    PARAM.int_value("MESH_NOISED") != 0);
+  base_small_vector tt(N); tt[1] = -0.5;
+  mesh.translation(tt); 
+  
+
+  scalar_type refinement_radius;
+  refinement_radius = PARAM.real_value("REFINEMENT_RADIUS","Refinement Radius");
+  size_type refinement_process;
+  refinement_process = PARAM.int_value("REFINEMENT_PROCESS","Refinement process");
+  dal::bit_vector conv_to_refine;
+  size_type ref = 1;  
+  if (refinement_radius > 0){
+    while(ref <= refinement_process){
+      conv_to_refine.clear();
+      for(dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i){
+      for(size_type j=0; j < 3; ++j)
+	if(fabs(mesh.points()[mesh.ind_points_of_convex(i)[j]][0])<refinement_radius 
+	   && fabs(mesh.points()[mesh.ind_points_of_convex(i)[j]][1])<refinement_radius){
+	  conv_to_refine.add(i);
+	}
+      }
+      mesh.Bank_refine(conv_to_refine);
+      
+      ref = ref + 1; 
+      refinement_radius = refinement_radius/3.;
+      if(refinement_radius > 1e-16 )
+	cout<<"refining process step " << ref << "... refining "<< conv_to_refine.size() <<" convexes..." << endl ; 
+      
+    }
+  cout<<"refining process complete." << endl ;
+  }
+
+
+
+  datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
+  residual = PARAM.real_value("RESIDUAL");
+  if (residual == 0.) residual = 1e-10;
+  enr_area_radius = PARAM.real_value("RADIUS_ENR_AREA",
+				     "radius of the enrichment area");
+  
+  GLOBAL_FUNCTION_MF = PARAM.string_value("GLOBAL_FUNCTION_MF");
+  GLOBAL_FUNCTION_U = PARAM.string_value("GLOBAL_FUNCTION_U");
+
+  reference_test = PARAM.int_value("REFERENCE_TEST", "Reference test"); 
+
+  unsigned EXACT_SOL_NUM = PARAM.int_value("EXACT_SOL_NUM", "Exact solution function number"); 
+  
 
   cutoff_func = PARAM.int_value("CUTOFF_FUNC", "cutoff function");
 
@@ -608,7 +418,7 @@ void crack_problem::init(void) {
   
   
 #ifdef VALIDATE_XFEM
-  exact_sol.init(1, lambda, mu, ls);
+  exact_sol.init(EXACT_SOL_NUM, lambda, mu, ls);
 #endif
 }
 
@@ -740,18 +550,20 @@ bool crack_problem::solve(plain_vector &U) {
   
   mf_sing_u.set_functions(vfunc);
   
-  
-  if (enrichment_option == 3 || enrichment_option == 4) {
-    spider = new getfem::spider_fem(spider_radius, mim, spider_Nr,
-				    spider_Ntheta, spider_K, translation,
-				    theta0);
-    mf_us.set_finite_element(mesh.convex_index(),spider->get_pfem());
+
+  if (enrichment_option == SPIDER_FEM_ALONE || 
+      enrichment_option == SPIDER_FEM_ENRICHMENT) {
+    spider.fem = new getfem::spider_fem(spider.radius, mim, spider.Nr,
+					spider.Ntheta, spider.K, translation,
+					spider.theta0, spider.bimat_enrichment,
+					spider.epsilon);
+    mf_us.set_finite_element(mesh.convex_index(),spider.fem->get_pfem());
     for (dal::bv_visitor_c i(mf_us.convex_index()); !i.finished(); ++i) {
       if (mf_us.fem_of_element(i)->nb_dof(i) == 0) {
 	mf_us.set_finite_element(i,0);
       }
     }
-    spider->check();
+    spider.fem->check();
   }
 
   switch (enrichment_option) {
@@ -942,9 +754,9 @@ bool crack_problem::solve(plain_vector &U) {
   if(bimaterial ==  1){
     //down side
     for(size_type i = 1; i<F.size(); i=i+2) 
-      F[i] = -0.2;
+      F[i] = -0.4;
     for(size_type i = 0; i<F.size(); i=i+2) 
-      F[i] = -0;
+      F[i] = -0.2;
   }
   
   getfem::mdbrick_source_term<>  NEUMANN(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM);   
@@ -953,18 +765,18 @@ bool crack_problem::solve(plain_vector &U) {
   for(size_type i = 1; i<F.size(); i=i+2) 
     F[i] = 0.;
   for(size_type i = 0; i<F.size(); i=i+2) 
-    F[i] = -0.1;
+    F[i] = -0.;
   getfem::mdbrick_source_term<> NEUMANN_HOM(NEUMANN, mf_rhs, F,NEUMANN_HOMOGENE_BOUNDARY_NUM);
    
     //upper side
   gmm::clear(F);
   for(size_type i = 1; i<F.size(); i=i+2) 
-    F[i] = 0.2;
+    F[i] = 0.4;
   for(size_type i = 0; i<F.size(); i=i+2) 
-    F[i] = -0.2;
+    F[i] = 0.;
   getfem::mdbrick_source_term<> NEUMANN1(NEUMANN_HOM, mf_rhs, F,NEUMANN_BOUNDARY_NUM1);
   
-  if (bimaterial ==1)
+  if (bimaterial == 1)
     pNEUMANN = & NEUMANN1;
   else
     pNEUMANN = & NEUMANN;
@@ -1066,14 +878,15 @@ bool crack_problem::solve(plain_vector &U) {
   cout << "Total number of variables : " << final_model->nb_dof() << endl;
   getfem::standard_model_state MS(*final_model);
   gmm::iteration iter(residual, 1, 40000);
+  cout << "Solving..." << endl;
   getfem::standard_solve(MS, *final_model, iter);
-  
+  cout << "Solving... done" << endl;
   // Solution extraction
   gmm::copy(ELAS.get_solution(MS), U);
   
   if(reference_test)
     {
-      cout << "Reference solution export...";
+      cout << "Exporting reference solution...";
       dal::bit_vector blocked_dof = mf_u().dof_on_set(5);
       getfem::mesh_fem mf_refined(mesh, N);
       std::string FEM_DISC = PARAM.string_value("FEM_DISC","fem disc ");
@@ -1093,6 +906,33 @@ bool crack_problem::solve(plain_vector &U) {
   return (iter.converged());
 }
 
+void export_interpolated_on_line(const getfem::mesh_fem &mf,
+				 const getfem::base_vector &U,
+				 const base_node &x0,
+				 const base_small_vector &dir,
+				 const int nb_points, 
+				 const std::string &filename) {
+  getfem::mesh_trans_inv mti(mf.linked_mesh());
+  scalar_type h = 1.0/(2*nb_points);
+  for (int i=-nb_points; i <= nb_points; ++i) {
+    mti.add_point(x0 + 2*(i*h)*dir);
+  }
+
+   getfem::base_vector V(mti.nb_points() * mf.get_qdim());
+  getfem::base_matrix M;
+  getfem::interpolation(mf, mti, U, V, M, 0, false);
+  
+  std::ofstream f(filename.c_str()); f.precision(16);
+  
+  for (size_type i=0; i < mti.nb_points(); ++i) {
+    for (unsigned q=0; q < mf.get_qdim(); ++q) {
+      f << V[i*mf.get_qdim()+q] << " ";
+    }
+    f << "\n";
+  }
+}
+				 
+
 /**************************************************************************/
 /*  main program.                                                         */
 /**************************************************************************/
@@ -1140,7 +980,7 @@ int main(int argc, char *argv[]) {
       if (NX < 6) nn = 24;
       else if (NX < 12) nn = 6;
       else if (NX < 30) nn = 3;
-      else nn = 1;
+      else nn = 3;
 
       /* choose an adequate slice refinement based on the distance to the crack tip */
       std::vector<bgeot::short_type> nrefine(mcut.convex_index().last_true()+1);
@@ -1177,7 +1017,7 @@ int main(int argc, char *argv[]) {
 					 ("IM_TRIANGLE(6)"));
 
       getfem::mesh_fem mf_refined(mcut_refined, Q);
-      mf_refined.set_classical_discontinuous_finite_element(2, 0.0001);
+      mf_refined.set_classical_discontinuous_finite_element(2, 0.001);
       plain_vector W(mf_refined.nb_dof());
 
       getfem::interpolation(p.mf_u(), mf_refined, U, W);
@@ -1195,7 +1035,7 @@ int main(int argc, char *argv[]) {
 
       if (p.PARAM.int_value("VTK_EXPORT")) {
 	getfem::mesh_fem mf_refined_vm(mcut_refined, 1);
-	mf_refined_vm.set_classical_discontinuous_finite_element(1, 0.0001);
+	mf_refined_vm.set_classical_discontinuous_finite_element(1, 0.001);
 	cerr << "mf_refined_vm.nb_dof=" << mf_refined_vm.nb_dof() << "\n";
 	plain_vector VM(mf_refined_vm.nb_dof());
 
@@ -1221,7 +1061,14 @@ int main(int argc, char *argv[]) {
 	exp.write_point_data(mf_refined_vm, VM, "von mises stress");
 
 	exp.write_point_data(mf_refined, W, "elastostatic_displacement");
-      
+
+	base_node line_x0(0.70001,0);
+	base_small_vector line_dir(0, 0.5001);
+	unsigned line_nb_points = 1000;
+	export_interpolated_on_line(mf_refined_vm, VM, 
+				    line_x0, line_dir, line_nb_points,
+				    "von_mises_on_line.data");
+
 #ifdef VALIDATE_XFEM
 
 	plain_vector VM_EXACT(mf_refined_vm.nb_dof());
@@ -1243,6 +1090,9 @@ int main(int argc, char *argv[]) {
 	exp2.write_point_data(mf_refined_vm, VM_EXACT, "exact von mises stress");
 	exp2.write_point_data(mf_refined, EXACT, "reference solution");
 	
+	export_interpolated_on_line(mf_refined_vm, VM_EXACT, 
+				    line_x0, line_dir, line_nb_points,
+				    "von_mises_on_line_exact.data");
 #endif
 	
 	cout << "export done, you can view the data file with (for example)\n"
@@ -1266,7 +1116,7 @@ int main(int argc, char *argv[]) {
 	plain_vector ref_U(ref_mf.nb_dof());
       
 	for (unsigned i=0; i < ref_mf.nb_dof(); ++i) {
-	  f >> ref_U[i]; if (!f.good()) DAL_THROW(gmm::failure_error, "problem while reading " << REFERENCE_U);
+	  f >> ref_U[i]; if (!f.good()) GMM_ASSERT1(f.good(), "problem while reading " << REFERENCE_U);
 	}
 	
 	getfem::mesh_im ref_mim(ref_m);
@@ -1278,12 +1128,12 @@ int main(int argc, char *argv[]) {
 	
 	//gmm::add(gmm::scaled(interp_U, -1), ref_U);
 	  
-
-	cout << "TO ref L2 ERROR:"<< getfem::asm_L2_dist(ref_mim, ref_mf, interp_U,
-						  ref_mf, ref_U)
-	     << endl << "To ref H1 ERROR:"
-	     << getfem::asm_H1_dist(ref_mim, ref_mf, interp_U,
-				    ref_mf, ref_U) << "\n";
+	
+	cout << "To ref L2 ERROR:" << getfem::asm_L2_dist(ref_mim, ref_mf, interp_U,
+							  ref_mf, ref_U) << endl;
+	
+	cout << "To ref H1 ERROR:" << getfem::asm_H1_dist(ref_mim, ref_mf, interp_U,
+							  ref_mf, ref_U) << endl;
 	
 	//cout << "L2 ERROR:"<< getfem::asm_L2_norm(ref_mim, ref_mf, ref_U)
 	//     << endl << "H1 ERROR:"
@@ -1291,17 +1141,22 @@ int main(int argc, char *argv[]) {
 	
       }
       
-      
 #ifdef VALIDATE_XFEM
-      cout << "L2 ERROR:"<< getfem::asm_L2_dist(p.mim, p.mf_u(), U,
-						p.exact_sol.mf, p.exact_sol.U)
-	   << endl << "H1 ERROR:"
-	   << getfem::asm_H1_dist(p.mim, p.mf_u(), U,
-	   			  p.exact_sol.mf, p.exact_sol.U) << "\n";
 
-      // cout << "Relative L2 Error"  << getfem::asm_L2_norm(p.mim,p.mf_u(),U)<<endl;
-      //cout << "Relative L2 Error"  << getfem::asm_H1_norm(p.mim,p.mf_u(),U)<<endl;
+      else {
+	cout << "L2 ERROR:"<< getfem::asm_L2_dist(p.mim, p.mf_u(), U,
+						  p.exact_sol.mf, p.exact_sol.U)
+	     << endl << "H1 ERROR:"
+	     << getfem::asm_H1_dist(p.mim, p.mf_u(), U,
+				    p.exact_sol.mf, p.exact_sol.U) << "\n";
+	
+      }
+
+      cout << "L2 norm of the solution:"  << getfem::asm_L2_norm(p.mim,p.mf_u(),U)<<endl;
+      cout << "H1 norm of the solution:"  << getfem::asm_H1_norm(p.mim,p.mf_u(),U)<<endl;
       
+      
+
       /* cout << "OLD ERROR L2:" 
 	 << getfem::asm_L2_norm(mim_refined,mf_refined,DIFF) 
 	 << " H1:" << getfem::asm_H1_dist(mim_refined,mf_refined,
