@@ -324,7 +324,21 @@ void bilaplacian_crack_problem::init(void) {
 				     "radius of the enrichment area");
     
     /* First step : build the mesh */
-    
+
+ if (!MESH_FILE.empty()) {
+    mesh.read_from_file(MESH_FILE);
+    base_small_vector tt(N); 
+    tt[0] = PARAM.real_value("TRANSLAT_X") ;  
+    tt[1] = PARAM.real_value("TRANSLAT_Y") ; 
+    cout << "TRANSLAT_X = " << tt[0] << " ; TRANSLAT_Y = " << tt[1] << "\n" ;
+    mesh.translation(tt); 
+    MESH_TYPE = bgeot::name_of_geometric_trans
+      (mesh.trans_of_convex(mesh.convex_index().first_true()));
+    bgeot::pgeometric_trans pgt = 
+      bgeot::geometric_trans_descriptor(MESH_TYPE);
+    cout << "MESH_TYPE=" << MESH_TYPE << "\n";
+    N = mesh.dim();
+ } else {
     bgeot::pgeometric_trans pgt = 
       bgeot::geometric_trans_descriptor(MESH_TYPE);
     N = pgt->dim();
@@ -343,14 +357,33 @@ void bilaplacian_crack_problem::init(void) {
     mesh.transformation(M);
     
     base_small_vector tt(N); 
-    tt[0] = -0.5 ;
-    tt[1] = -0.5;
+    tt[0] = - 0.5 + PARAM.real_value("TRANSLAT_X") ; 
+    tt[1] = - 0.5 + PARAM.real_value("TRANSLAT_Y") ; 
     mesh.translation(tt); 
-  
-    scalar_type quality = 1.0;
-    for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i)
+ }    
+ 
+    scalar_type quality = 1.0, avg_area = 0. , min_area = 1. , max_area = 0., area ;
+    scalar_type radius, avg_radius = 0., min_radius = 1., max_radius = 0. ;
+    size_type cpt = 0 ; 
+    for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i){
     	quality = std::min(quality, mesh.convex_quality_estimate(i));
+	area = mesh.convex_area_estimate(i) ;
+	radius = mesh.convex_radius_estimate(i) ;
+	avg_area += area ;
+	avg_radius += radius ;
+	min_radius = std::min(radius, min_radius) ;
+	max_radius = std::max(radius, max_radius) ;	
+	min_area = std::min(min_area, area) ; 
+	max_area = std::max(max_area, area) ;
+	cpt++ ;
+    }
+    avg_area /= cpt ;
+    avg_radius /= cpt ; 
     cout << "quality of mesh : " << quality << endl;
+    cout << "average radius : " << avg_radius << endl;
+    cout << "radius min : " << min_radius << " ; radius max : " << max_radius << endl;
+    cout << "average area : " << avg_area << endl ;
+    cout << "area min : " << min_area << " ; area max : " << max_area << endl;        
     
    /* read the parameters   */
   epsilon = PARAM.real_value("EPSILON", "thickness") ;
@@ -360,7 +393,7 @@ void bilaplacian_crack_problem::init(void) {
   mortar_version = getfem::constraints_type(mv);
   datafilename=PARAM.string_value("ROOTFILENAME","Base name of data files.");
   residual=PARAM.real_value("RESIDUAL"); if (residual == 0.) residual = 1e-10;
-  KL = (PARAM.int_value("KL", "Kirchhoff-Love model or not") != 0);
+  KL = PARAM.int_value("KL", "Kirchhoff-Love model or not") != 0;
   D = PARAM.real_value("D", "Flexion modulus");
   if (KL) nu = PARAM.real_value("NU", "Poisson ratio");
 
@@ -382,10 +415,6 @@ void bilaplacian_crack_problem::init(void) {
   mf_pre_u.set_finite_element(mesh.convex_index(), pf_u); 
   getfem::pfem pf_partition_of_unity = getfem::fem_descriptor(PARAM.string_value("PARTITION_OF_UNITY_FEM_TYPE")) ; 
   mf_partition_of_unity.set_finite_element(mesh.convex_index(), pf_partition_of_unity);     
-  mf_pre_mortar.set_finite_element(mesh.convex_index(),
-             getfem::fem_descriptor(PARAM.string_value("MORTAR_FEM_TYPE")));
-  mf_pre_mortar_deriv.set_finite_element(mesh.convex_index(),
-             getfem::fem_descriptor(PARAM.string_value("MORTAR_DERIV_FEM_TYPE")));
   
   // set the mesh_fem of the multipliers (for the dirichlet condition)    
   std::string dirichlet_fem_name = PARAM.string_value("DIRICHLET_FEM_TYPE");
@@ -397,7 +426,7 @@ void bilaplacian_crack_problem::init(void) {
 			       getfem::fem_descriptor(dirichlet_fem_name));
   }
   std::string dirichlet_der_fem_name
-    = PARAM.string_value("DIRICHLET_DER_FEM_TYPE", "");
+    = PARAM.string_value("DIRICHLET_DER_FEM_TYPE", "");  // for the dirichlet condition on the derivative
   if (dirichlet_der_fem_name.size() == 0)
     mf_mult_d.set_finite_element(mesh.convex_index(), pf_u);
   else {
@@ -418,7 +447,13 @@ void bilaplacian_crack_problem::init(void) {
     mf_rhs.set_finite_element(mesh.convex_index(), 
 			      getfem::fem_descriptor(data_fem_name));
   }
-  
+
+  // set the mesh_fem for the multipliers for the case the Integral Matching  
+  mf_pre_mortar.set_finite_element(mesh.convex_index(),
+             getfem::fem_descriptor(PARAM.string_value("MORTAR_FEM_TYPE")));
+  mf_pre_mortar_deriv.set_finite_element(mesh.convex_index(),
+             getfem::fem_descriptor(PARAM.string_value("MORTAR_DERIV_FEM_TYPE")));
+	     
   /* set boundary conditions
    * (Neuman on the upper face, Dirichlet elsewhere) */
   cout << "Selecting Neumann and Dirichlet boundaries\n";
@@ -437,29 +472,29 @@ void bilaplacian_crack_problem::init(void) {
 void bilaplacian_crack_problem::compute_error(plain_vector &U) {
 
   if (PARAM.real_value("RADIUS_SPLIT_DOMAIN") == 0){
-// plain_vector V(gmm::vect_size(U)) ;
-// gmm::clear(V) ;
-//   cout << "L2 ERROR : "
-//        << getfem::asm_L2_dist(mim, mf_u(), V,
-// 			      exact_sol.mf, exact_sol.U) << "\n";
-//   cout << "H1 ERROR : "
-//        << getfem::asm_H1_dist(mim, mf_u(), V,
-// 			      exact_sol.mf, exact_sol.U) << "\n";
-//   cout << "H2 ERROR : "
-//        << getfem::asm_H2_dist(mim, mf_u(), V, 
-//                               exact_sol.mf, exact_sol.U) << "\n"; 
+     plain_vector V(gmm::vect_size(U)) ;
+     gmm::clear(V) ;
 
-  cout << "L2 ERROR : "
+    cout << "\nL2 ERROR : "
        << getfem::asm_L2_dist(mim, mf_u(), U,
 			      exact_sol.mf, exact_sol.U) << "\n";
-  if (PARAM.int_value("ASM_H1_AND_H2_DIST") == 1){
     cout << "H1 ERROR : "
          << getfem::asm_H1_dist(mim, mf_u(), U,
   	  		      exact_sol.mf, exact_sol.U) << "\n";
     cout << "H2 ERROR : "
          << getfem::asm_H2_dist(mim, mf_u(), U, 
                               exact_sol.mf, exact_sol.U) << "\n"; 
-  }
+    if ( PARAM.int_value("NORM_EXACT") ){
+    cout << "L2 exact : "
+         << getfem::asm_L2_dist(mim, mf_u(), V,
+			      exact_sol.mf, exact_sol.U) << "\n";
+    cout << "H1 exact : "
+         << getfem::asm_H1_dist(mim, mf_u(), V,
+			      exact_sol.mf, exact_sol.U) << "\n";
+    cout << "H2 exact : "
+         << getfem::asm_H2_dist(mim, mf_u(), V, 
+                              exact_sol.mf, exact_sol.U) << "\n";
+    } 
   }
   else {
   getfem::mesh_region r_center, r_ext ;
@@ -516,16 +551,15 @@ void bilaplacian_crack_problem::compute_error(plain_vector &U) {
 /*  Model.                                                                */
 /**************************************************************************/
 
-bool bilaplacian_crack_problem::solve(plain_vector &U) {
+bool bilaplacian_crack_problem::solve(plain_vector &U) {  
   size_type nb_dof_rhs = mf_rhs.nb_dof();
-  //size_type N = mesh.dim();
   
   // Setting the level-set
   ls.reinit();  
   for (size_type d = 0; d < ls.get_mesh_fem().nb_dof(); ++d) {
     scalar_type x = ls.get_mesh_fem().point_of_dof(d)[0];
     scalar_type y = ls.get_mesh_fem().point_of_dof(d)[1];
-    ls.values(0)[d] = y + 1/4.*(x + .25);
+    ls.values(0)[d] = y  ; // + 1/4.*(x + .25);
     ls.values(1)[d] = x;
   }
   //ls.simplify(0.5);
@@ -564,9 +598,8 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
        for (unsigned j=0; j < mesh.nb_points_of_convex(i); ++j) {
 	 if (gmm::sqr(mesh.points_of_convex(i)[j][0]) + 
 	     gmm::sqr(mesh.points_of_convex(i)[j][1]) > 
-	     gmm::sqr(enr_area_radius)) {
+	     gmm::sqr(enr_area_radius)) 
 	   pm_convexes.sup(i); break;
-	 }
        }
      }
       
@@ -598,23 +631,22 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
             if (gmm::sqr(X[j]) + gmm::sqr(Y[j]) <= gmm::sqr(enr_area_radius))
 	           enriched_dofs.add(j);
           }
-          cout << "enriched_dofs: " << enriched_dofs << "\n";
-          if (enriched_dofs.card() < 3)
-             GMM_WARNING0("There is " << enriched_dofs.card() <<
+      cout << "enriched_dofs: " << enriched_dofs << "\n";
+      if (enriched_dofs.card() < 3)
+            GMM_WARNING0("There is " << enriched_dofs.card() <<
 		   " enriched dofs for the crack tip");
-          mf_u_product.set_enrichment(enriched_dofs);
-          mf_u_sum.set_mesh_fems(mf_u_product, mfls_u);
-          cout << "enrichment done \n" ;}
-	  break ;
+      mf_u_product.set_enrichment(enriched_dofs);
+      mf_u_sum.set_mesh_fems(mf_u_product, mfls_u);
+      cout << "enrichment done \n" ;}
+      break ;
   case 3 : // Integral matching (mortar)
     {
-    cout << "\nIntegral Matching (Mortar)\n" ;
-    
-    dal::bit_vector cvlist_in_area;
-    dal::bit_vector cvlist_out_area;
+    cout << "\nIntegral Matching (Mortar)\n" ;    
+    dal::bit_vector cvlist_in_area, cvlist_out_area;
+    bool in_area = true;
     for (dal::bv_visitor cv(mesh.convex_index()); 
 	   !cv.finished(); ++cv) {
-	bool in_area = true;
+	in_area = true;
 	/* For each element, we test all of its nodes. 
 	   If all the nodes are inside the enrichment area,
 	   then the element is completly inside the area too */ 
@@ -622,7 +654,8 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	  if (gmm::sqr(mesh.points_of_convex(cv)[j][0] ) + 
 	      gmm::sqr(mesh.points_of_convex(cv)[j][1] ) > 
 	      gmm::sqr(enr_area_radius)) {
-	    in_area = false; break;
+	          in_area = false; 
+		  break;
 	  }
 	}
 
@@ -659,12 +692,32 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
       cout << "MORTAR_BOUNDARY_IN: " << mesh.region(MORTAR_BOUNDARY_IN) << "\n";
       cout << "MORTAR_BOUNDARY_OUT: " << mesh.region(MORTAR_BOUNDARY_OUT) << "\n";
       
-      // Searching the element that is both crossed by the crack
-      // and with one of his border which constitutes a part of the 
+      // an optional treatment : creating a representation of the enrichment area     
+      getfem::mesh_fem mf_enrich(mesh);
+      getfem::pfem pf_mef = getfem::classical_fem(mesh.trans_of_convex(mesh.convex_index().first_true()), 1 );
+      mf_enrich.set_finite_element(mesh.convex_index(), pf_mef) ;
+      std::vector<scalar_type> UU(mf_enrich.nb_dof()) ;
+      std::fill(UU.begin(), UU.end() ,0.) ;
+      cout << "exporting the enrichment zone: \n" ;
+      for (dal::bv_visitor i(cvlist_in_area) ; !i.finished() ; ++i){ 
+	  for (unsigned int j = 0 ; j < mf_enrich.ind_dof_of_element(i).size() ; ++j )  
+	  UU[mf_enrich.ind_dof_of_element(i)[j]] = 1. ;         
+      }
+      
+      cout << "exporting enrichment to " << "enrichment_zone.vtk" << "..\n";
+      getfem::vtk_export exp("enrichment_zone.vtk", false);
+      exp.exporting(mf_enrich); 
+      exp.write_point_data(mf_enrich, UU, "enrichment");
+      cout << "export done, you can view the data file with (for example)\n"
+	"mayavi -d enrichment_zone.vtk -f "
+	"WarpScalar -m BandedSurfaceMap -m Outline\n";
+	
+      // Another optional treatment :
+      // Searching the elements that are both crossed by the crack
+      // and with one of their faces which constitutes a part of the 
       // boundary between the enriched zone and the rest of the domain.
       getfem::mesh_region &boundary = mesh.region(MORTAR_BOUNDARY_IN);
-      
-      unsigned cpt = 0 ;
+      unsigned int cpt = 0 ;
       for (dal::bv_visitor i(cvlist_in_area); !i.finished(); ++i) {
          if (mls.is_convex_cut(i)){
 	    // Among the faces of the convex, we search if some are
@@ -674,15 +727,14 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	        if (boundary.is_in(i,j))
 		   cpt += 1;
 	    }
-	 if (cpt) {
-            cout << "\n The convex number " << i << " is crossed by the crack :\n" ;
-	    cout << "  it has : " << cpt << " face(s) among the boundary.\n \n " ;
+	    if (cpt) {
+               cout << "\n The convex number " << i << " is crossed by the crack :\n" ;
+	       cout << "  it has : " << cpt << " face(s) among the boundary.\n \n " ;
+	    }
 	 }
-	 }
-	 
       }
  
-    }
+    }  // end of "enrichment_option = 3"
     break ;
   default : 
 	GMM_ASSERT1(false, "Enrichment_option parameter is undefined");
@@ -690,7 +742,7 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	}
   mesh.write_to_file("toto.mesh");
   
-  if (0) {
+  if (0) {  // printing the type of each dof
     unsigned Q = mf_u().get_qdim();
     for (unsigned d=0; d < mf_u().nb_dof(); d += Q) {
       printf("dof %4d @ %+6.2f:%+6.2f: ", d, 
@@ -756,8 +808,9 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
   NDER_DIRICHLET.set_constraints_type(dirichlet_version);
   NDER_DIRICHLET.R_must_be_derivated(); // hence we give the exact solution , and its gradient will be taken
   NDER_DIRICHLET.rhs().set(exact_sol.mf,exact_sol.U);
-
-
+  if (dirichlet_version == getfem::PENALIZED_CONSTRAINTS)
+    NDER_DIRICHLET.set_penalization_parameter(PARAM.real_value("EPS_DIRICHLET_PENAL")) ;
+  
   // Defining the Dirichlet condition value.
   gmm::resize(F, nb_dof_rhs);
   getfem::interpolation_function(mf_rhs, F, sol_u,SIMPLE_SUPPORT_BOUNDARY_NUM);
@@ -790,7 +843,11 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
     getfem::base_vector R(mf_mortar.nb_dof());
       
     if (mortar_type == 1) {
-      // older version of integral matching (jan-feb 2007)      
+      /* older version of integral matching (jan-feb 2007) :
+       *
+       *  \int_Gamma (u-v) \lambda + \partial_n ( u + v) \partial_n \lambda = 0, for all \lambda in \Lambda      
+      */
+ 
       /* build the list of dof for the "(u-v) lambda" and for the  "\partial_n(u-v) \partial_n lambda" term in the mortar condition */  
       dal::bit_vector bv_mortar;
       dal::bit_vector bv_deriv;
@@ -804,7 +861,7 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
       
       gmm::resize(R, gmm::mat_nrows(MD));
       asm_normal_derivative_dirichlet_constraints_bis(MD, R, mim, mf_mortar,
-                                                      mf_mortar, mf_pre_mortar, R, MORTAR_BOUNDARY_OUT, 0, getfem::ASMDIR_BUILDH) ;
+            mf_mortar, mf_pre_mortar, R, MORTAR_BOUNDARY_OUT, 0, getfem::ASMDIR_BUILDH) ;
       
       for (size_type i=0; i < mf_mortar.nb_dof(); ++i) {
 	if ( (MM(i,i) > 1e-15) & (MD(i,i) > 1e-15) ) { 
@@ -821,9 +878,7 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	  bv_union.add(i);
 	}
       }
-      
-      //cout << "matrice des drives normales du mesh_fem_mortar : \n" << MD << "\n" ;
-      
+           
       for (dal::bv_visitor d(bv_mortar); !d.finished(); ++d)
 	ind_mortar.push_back(d);
       for (dal::bv_visitor d(bv_deriv); !d.finished(); ++d)
@@ -838,15 +893,11 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	 sometimes return too much dof when the mesh_fem is enriched. */
       
       
-      
-      
       /* building matrices */
       cout << "Handling mortar junction (" << ind_union.size() << 
-	" dof for the lagrange multiplier)\n";
-      
+	" dof for the lagrange multiplier)\n";    
       gmm::resize(H0, mf_mortar.nb_dof(), mf_u().nb_dof()) ;
       gmm::resize(H,  ind_union.size(),   mf_u().nb_dof()) ;
-      
       cout << "bv_mortar = " << bv_mortar << "\n";
       cout << "bv_deriv = " << bv_deriv << "\n" ;
       cout << "bv_union = " << bv_union << "\n" ;
@@ -888,9 +939,92 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
       
       // -----------------------------------
     }
-    else{
+    if (mortar_type == 3 ) {
+      /* other version of the integral matching :
+       *     \int_Gamma        (u-v) \lambda  = 0, for all \lambda in \Lambda
+       *     \int_Gamma \partial_n (u-v)\mu  = 0, for all \mu in M
+      */
+      
+      /* build the list of dof for the "(u-v) lambda" condition  */  
+      dal::bit_vector bv_mortar;
+      dal::bit_vector bv_deriv;
+      sparse_matrix MM(mf_mortar.nb_dof(), mf_mortar.nb_dof());
+      sparse_matrix MD(mf_mortar_deriv.nb_dof(), mf_mortar_deriv.nb_dof());
+      std::vector<size_type> ind_mortar;
+      std::vector<size_type> ind_deriv;
+      getfem::asm_mass_matrix(MM, mim, mf_mortar, MORTAR_BOUNDARY_OUT);
+      getfem::asm_mass_matrix(MD, mim, mf_mortar_deriv, MORTAR_BOUNDARY_OUT);
+
+      for (size_type i=0; i < mf_mortar.nb_dof(); ++i)
+	if (MM(i,i) > 1e-15) { 
+	  // cout << "dof " << i << " MM = " << MM(i,i) << endl;
+	  bv_mortar.add(i); ind_mortar.push_back(i);
+	}
+      for (size_type i=0; i < mf_mortar_deriv.nb_dof(); ++i)
+	if (MD(i,i) > 1e-15) { bv_deriv.add(i); ind_deriv.push_back(i); }
+      
+      // building matrices
+      
+      cout << "Handling mortar junction (" << ind_mortar.size() << 
+	" dof for the lagrange multiplier of the displacement, " <<
+	ind_deriv.size() << " dof for the lagrange multiplier of the derivative)\n";
+      
+      gmm::resize(H0, mf_mortar.nb_dof(), mf_u().nb_dof()) ;
+      gmm::resize(H,  ind_mortar.size() + ind_deriv.size(), mf_u().nb_dof()) ; 
+      
+      // Defining sub_indexes of the matrices calculated with the 
+      // complete set of dofs.
+      gmm::sub_index sub_i(ind_mortar);
+      gmm::sub_index sub_i1(ind_deriv);
+      gmm::sub_interval sub_j(0, mf_u().nb_dof());
+      
+      gmm::sub_interval sub_val_H(0, ind_mortar.size()) ;
+      gmm::sub_interval sub_deriv_H(ind_mortar.size(), ind_deriv.size()) ;
+      
+      cout << "sub_indexes built\n" ;
+      /* build the mortar constraint matrix -- note that the integration
+	 method is conformal to the crack
+      */
+      gmm::clear(H0);
+      getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), MORTAR_BOUNDARY_OUT);
+      gmm::copy(gmm::sub_matrix(H0, sub_i, sub_j), gmm::sub_matrix(H, sub_val_H, sub_j) );
     
-      // for the condition \int_\Gamma \mu \nabla \u = 0, with \mu vectorial
+      gmm::clear(H0);
+      getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), MORTAR_BOUNDARY_IN);
+      gmm::add(gmm::scaled(gmm::sub_matrix(H0, sub_i, sub_j), -1), gmm::sub_matrix(H, sub_val_H, sub_j) );
+      
+      
+      cout << "first contraint asm\n" ;
+      gmm::resize(R, ind_deriv.size());
+      gmm::clear(H0);
+      gmm::resize(H0, mf_mortar_deriv.nb_dof(), mf_u().nb_dof() ) ;
+
+      getfem::asm_normal_derivative_dirichlet_constraints
+	(H0, R, mim, mf_u(),
+	 mf_mortar_deriv, mf_pre_u, R, MORTAR_BOUNDARY_OUT, 0, getfem::ASMDIR_BUILDH) ;
+
+      gmm::add(gmm::sub_matrix(H0, sub_i1, sub_j),
+	       gmm::sub_matrix(H, sub_deriv_H, sub_j)) ;
+      
+      cout << "first step \n" ;
+      gmm::clear(H0);
+
+      getfem::asm_normal_derivative_dirichlet_constraints
+	(H0, R, mim, mf_u(),
+	 mf_mortar_deriv, mf_pre_u, R, MORTAR_BOUNDARY_IN, 0, getfem::ASMDIR_BUILDH) ;
+      gmm::add(gmm::scaled(gmm::sub_matrix(H0, sub_i1, sub_j), +1.0),
+	       gmm::sub_matrix(H, sub_deriv_H, sub_j)) ;
+      
+    }
+
+    if (mortar_type == 2 ) {
+      /* other version of the integral matching :
+       *     \int_Gamma        (u-v) \lambda  = 0, for all \lambda in \Lambda
+       *     \int_Gamma \nabla (u-v).\mu      = 0, for all \mu in M    (be carefull : \mu is vectorial.
+      */
+      
+      
+      // for the condition \int_\Gamma \mu \nabla \u = 0, with \mu vectorial :
       mf_mortar_deriv.set_qdim(2) ;
       
       /* New version of the integral matching -----------------------------------------------*/
@@ -974,8 +1108,6 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
       
       // ------------------------------------------ end of new version
     }
-    
-    
     /* because of the discontinuous partition of mf_u(), some levelset 
        enriched functions do not contribute any more to the
        mass-matrix (the ones which are null on one side of the
@@ -984,6 +1116,7 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
     sparse_matrix M2(mf_u().nb_dof(), mf_u().nb_dof());
     getfem::asm_mass_matrix(M2, mim, mf_u(), mf_u());
     //gmm::HarwellBoeing_IO::write("M2.hb", M2);
+    cout << "PARAM.real_value(\"SEUIL\") : " << PARAM.real_value("SEUIL") << "\n" ;
     for (size_type d = 0; d < mf_u().nb_dof(); ++d) {
       //       if (M2(d,d) < 1e-7) cout << "  weak mf_u() dof " << d << " @ " << 
       // 	  mf_u().point_of_dof(d) << " M2(d,d) = " << M2(d,d) << "\n";
@@ -999,7 +1132,14 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
     final_model = &mortar;
     gmm::Harwell_Boeing_save("H.hb", H);        
     //------------------------------------------------------------------
-    // Matching of the normal derivative
+    // /* Matching of the normal derivative, another type of matching ;
+    //  * the second condition is replaced by a matching that only concerns the
+    //  * normal derivative :
+    //  *   \int_\Gamma \partial_n (u + v) \mu = 0, for all mu in M
+    //  *   (be carefull : \mu is scalar here) 
+    //  */
+    //  
+    //     mf_mortar_deriv.set_qdim(1) ;
     //     getfem::mdbrick_constraint<> &mortar_derivative = 
     //       *(new getfem::mdbrick_constraint<>(mortar,0));
     //       
@@ -1021,8 +1161,10 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
     //     mortar_derivative.set_constraints(H,R);
     //     final_model = &mortar_derivative ;
   }
+  // end of "if (enrichment_option == 3){ ... "
   
-  if (1) { // suppression of nodes with a very small term on the mass matrix diag
+  
+  if (0) { // suppression of nodes with a very small term on the stiffness matrix diag
     getfem::mdbrick_constraint<> &extra = *(new getfem::mdbrick_constraint<>(DIRICHLET, 0));
     extra.set_constraints_type(getfem::constraints_type(dirichlet_version));  
     if (dirichlet_version == getfem::PENALIZED_CONSTRAINTS)
@@ -1034,7 +1176,8 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
     base_vector RR(mf_rhs.nb_dof(), 1.0);
     getfem::asm_stiffness_matrix_for_bilaplacian(M2, mim, mf_u(), 
                                                  mf_rhs, RR);
-    
+    cout << "stiffness_matrix_for_bilaplacian : " << M2 << "\n" ;  
+    cout << "SEUIL = " << PARAM.real_value("SEUIL") << "\n" ;
     for (size_type d = 0; d < mf_u().nb_dof(); ++d) {
       if (M2(d,d) < PARAM.real_value("SEUIL")) {
 	cout << "OULALA " << d << " @ " << mf_u().point_of_dof(d) << " : " << M2(d,d) << "\n";	
@@ -1053,12 +1196,17 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
   cout << "Total number of variables : " << final_model->nb_dof() << endl;
   getfem::standard_model_state MS(*final_model);
   gmm::iteration iter(residual, 1, 40000);
-  getfem::standard_solve(MS, *final_model, iter);
+
+
+  // getfem::useful_types<getfem::standard_model_state>::plsolver_type p;
+  // p.reset(new getfem::linear_solver_cg_preconditioned_ildlt<sparse_matrix,plain_vector>);
+
+  getfem::standard_solve(MS, *final_model, iter /* , p*/);
 
   // Solution extraction
   gmm::resize(U, mf_u().nb_dof());
   gmm::copy(BIL.get_solution(MS), U);
-  return (iter.converged());
+  return true;
 }
 
 template<typename VEC1, typename VEC2>
