@@ -295,13 +295,15 @@ namespace bgeot {
      *  points in the reference convex. 
      */
     friend pgeotrans_precomp
-    geotrans_precomp(pgeometric_trans pg, pstored_point_tab ps);
+    geotrans_precomp(pgeometric_trans pg, pstored_point_tab ps,
+		     dal::pstatic_stored_object dep);
 
   };
 
 
   pgeotrans_precomp
-  geotrans_precomp(pgeometric_trans pg, pstored_point_tab ps);
+  geotrans_precomp(pgeometric_trans pg, pstored_point_tab ps,
+		   dal::pstatic_stored_object dep);
 
   template <typename CONT, typename VEC> 
   void geotrans_precomp_::transform(const CONT& G, size_type j,
@@ -312,6 +314,8 @@ namespace bgeot {
     for (typename CONT::const_iterator itk = G.begin(); 
          itk != G.end(); ++itk, ++k)
       gmm::add(gmm::scaled(*itk, c[j][k]), pt);
+    GMM_ASSERT1(k == pgt->nb_points(),
+		"Wrong number of points in tranformation");
   }
 
   template <typename CONT> 
@@ -336,8 +340,9 @@ namespace bgeot {
 
   public :
     
-    pgeotrans_precomp operator()(pgeometric_trans pg, pstored_point_tab pspt) {
-      pgeotrans_precomp p = geotrans_precomp(pg, pspt);
+    pgeotrans_precomp operator()(pgeometric_trans pg,
+				 pstored_point_tab pspt) {
+      pgeotrans_precomp p = geotrans_precomp(pg, pspt, 0);
       precomps.insert(p);
       return p;
     }
