@@ -217,6 +217,9 @@ namespace getfem {
   
   /** @brief Base class for finite element description */
   class virtual_fem : virtual public dal::static_stored_object {
+  public :
+    enum vec_type { VECTORIAL_NOTRANSFORM_TYPE, VECTORIAL_PRIMAL_TYPE,
+		    VECTORIAL_DUAL_TYPE };
 
   protected :
 
@@ -240,6 +243,8 @@ namespace getfem {
     bool real_element_defined;
     short_type es_degree; // estimated polynomial degree of the FEM
     short_type hier_raff; /// hierarchical refinement of the FEM
+    vec_type vtype; // for vectorial elements, type of transformation
+                    // from the reference element.
     std::string debug_name_;
     
   public :
@@ -268,6 +273,8 @@ namespace getfem {
     dim_type &dim(void) { return dim_; }
     /// dimension of the target space.
     dim_type target_dim(void) const { return ntarget_dim; }
+    /// Type of vectorial element.
+    vec_type vectorial_type(void) const { return vtype; }
     /// Return the convex of the reference element.
     virtual bgeot::pconvex_ref ref_convex(size_type) const { return cvr; }
     /// @internal
@@ -425,6 +432,7 @@ namespace getfem {
       is_equiv = is_pol = is_polycomp = is_lag = false;
       pspt_valid = false; hier_raff = 0; real_element_defined = false;
       es_degree = 5;
+      vtype = VECTORIAL_NOTRANSFORM_TYPE;
       cvs_node = bgeot::new_convex_structure();
     }
     virtual_fem(const virtual_fem& f) : dal::static_stored_object()
@@ -443,6 +451,7 @@ namespace getfem {
       cvr = f.cvr;
       dim_ = f.dim_;
       ntarget_dim = f.ntarget_dim;
+      vtype = f.vtype;
       is_equiv = f.is_equiv;
       is_lag = f.is_lag;
       is_pol = f.is_pol;
@@ -594,7 +603,8 @@ namespace getfem {
      If you need a set of "temporary" getfem::fem_precomp_, create
      them via a getfem::fem_precomp_pool structure. All memory will be
      freed when this structure will be destroyed.  */
-  pfem_precomp fem_precomp(pfem pf, bgeot::pstored_point_tab pspt);
+  pfem_precomp fem_precomp(pfem pf, bgeot::pstored_point_tab pspt,
+			   dal::pstatic_stored_object dep);
 
   /** Request for the removal of a pfem_precomp */
   inline void delete_fem_precomp(pfem_precomp pfp)
@@ -625,7 +635,7 @@ namespace getfem {
 	the fem_precomp_pool is destroyed.
     */
     pfem_precomp operator()(pfem pf, bgeot::pstored_point_tab pspt) {
-      pfem_precomp p = fem_precomp(pf, pspt);
+      pfem_precomp p = fem_precomp(pf, pspt, 0);
       precomps.insert(p);
       return p;
     }
