@@ -106,7 +106,8 @@ namespace getfem {
       virtual ~mesh_receiver() {}
   };
 
-  class approx_integration;
+  class integration_method;
+  typedef boost::intrusive_ptr<const integration_method> pintegration_method;
 
   /* ********************************************************************* */
   /*								   	   */
@@ -152,13 +153,11 @@ namespace getfem {
   */
 
   class mesh : virtual public dal::static_stored_object,
-		      public bgeot::basic_mesh,
-		      public context_dependencies {
+		       public bgeot::basic_mesh,
+		       public context_dependencies {
   public :
     
     typedef linkmsg_sender<mesh_receiver> msg_sender;
-    typedef gmm::approx_less<scalar_type> val_comp;
-    typedef bgeot::basic_mesh::pt_comp pt_comp;
     typedef bgeot::basic_mesh::PT_TAB PT_TAB;
     typedef bgeot::mesh_structure::ind_cv_ct ind_cv_ct;
     typedef bgeot::mesh_structure::ind_set ind_set;
@@ -183,7 +182,6 @@ namespace getfem {
     
     mutable bool cuthill_mckee_uptodate;
     mutable std::vector<size_type> cmk_order; // cuthill-mckee
-    scalar_type max_radius, characteristic_size;
     void init(void);
 
 #if GETFEM_PARA_LEVEL > 1
@@ -224,12 +222,12 @@ namespace getfem {
 #endif
     
     /// Constructor.
-    mesh(dim_type NN = dim_type(-1)); 
+    mesh(void); 
     mesh(const bgeot::basic_mesh &m);
     msg_sender &lmsg_sender(void) const { return lkmsg; }
     void update_from_context(void) const {}
     /// Mesh dimension.
-    dim_type dim(void) const { return dimension; }
+    dim_type dim(void) const { return pts.dim(); }
     /// Return the array of PT.
     const PT_TAB &points(void) const { return pts; }
     /// Return the array of PT.
@@ -260,25 +258,27 @@ namespace getfem {
 	already existing point.
 	@param pt the point coordinates.
     */
-    size_type add_point(const base_node &pt);
+    size_type add_point(const base_node &pt) { return pts.add_node(pt); }
     //			scalar_type characteristic_size = scalar_type(1));
     
     /// Give the number of geometrical nodes in the mesh.
     size_type nb_points(void) const { return pts.card(); }
     /// Return the points index
     const dal::bit_vector &points_index(void) const { return pts.index(); }
-    /// Delete the point of index i from the mesh.
-    void sup_point(size_type i) { if (!is_point_valid(i)) pts.sup(i); }
+    /** Delete the point of index i from the mesh if it is not linked to a
+	convex.
+    */
+    void sup_point(size_type i) { if (!is_point_valid(i)) pts.sup_node(i); }
     /// Swap the indexes of points of index i and j in the whole structure.
     void swap_points(size_type i, size_type j)
-    { if (i != j) { pts.swap(i,j); mesh_structure::swap_points(i,j); } }
+    { if (i != j) { pts.swap_points(i,j); mesh_structure::swap_points(i,j); } }
     /** Search a point given its coordinates.
 	@param pt the point that is searched.
 	@return the point index if pt was found in (or approximatively in)
 	the mesh nodes, size_type(-1) if not found.
     */
     size_type search_point(const base_node &pt) const
-    { return pts.search(pt); }
+    { return pts.search_node(pt); }
     /** Return the bgeot::geometric_trans attached to a convex.
 	@param ic the convex number.
     */
@@ -451,9 +451,9 @@ namespace getfem {
     /** Return an estimate of the convex smallest dimension. @see getfem::convex_radius_estimate */
     scalar_type minimal_convex_radius_estimate() const;
     /** Apply the given translation to each mesh node. */
-    void translation(base_small_vector);
+    void translation(const base_small_vector &);
     /** apply the given matrix transformation to each mesh node. */
-    void transformation(base_matrix);
+    void transformation(const base_matrix &);
     /** Return the bounding box [Pmin - Pmax] of the mesh. */
     void bounding_box(base_node& Pmin, base_node &Pmax) const;
     /** Return the region of index 'id'. Regions stored in mesh are
@@ -631,7 +631,7 @@ namespace getfem {
    */
   scalar_type convex_area_estimate(bgeot::pgeometric_trans pgt,
 				   const base_matrix& pts, 
-				   const approx_integration* pai);
+				   pintegration_method pim);
 
   /** rough estimate of the maximum value of the condition 
    * number of the jacobian of the geometric transformation */
