@@ -119,7 +119,7 @@ namespace getfem {
     Bank_info = 0;
   }
 
-  mesh::mesh(dim_type NN) : bgeot::basic_mesh(NN) { init(); }
+  mesh::mesh(void)  { init(); }
 
   mesh::mesh(const bgeot::basic_mesh &m) : bgeot::basic_mesh(m) { init(); }
 
@@ -181,36 +181,6 @@ namespace getfem {
   }
 #endif
 
-  size_type mesh::add_point(const base_node &pt) {
-    static scalar_type prec_factor = gmm::default_tol(scalar_type())
-      * scalar_type(10000);
-    scalar_type npt = gmm::vect_norm2(pt);
-    if (nb_points() == 0) {
-      dimension = pt.size();
-      characteristic_size = max_radius = npt;
-      pts.comparator().c.eps = characteristic_size * prec_factor;
-      return pts.add(pt);
-    }
-    else {
-      GMM_ASSERT1(pt.size() == dimension,
-		  "mesh::add_point : dimensions mismatch");
-      max_radius = std::max(max_radius, npt);
-
-      if (max_radius > scalar_type(10)*characteristic_size) {
-	characteristic_size = max_radius;
-	pts.comparator().c.eps = characteristic_size * prec_factor;
-	pts.resort();
-      }
-      
-      bgeot::basic_mesh::PT_TAB::const_sorted_iterator it = pts.sorted_ge(pt); 
-      if (it.index() != ST_NIL && pts.comparator()(pt, *it) == 0)
-	{ return it.index(); }
-      
-      size_type i = pts.add(pt);
-      return i;
-    }
-  }
-
   void mesh::optimize_structure() {
     size_type i, j;
     j = nb_convex();
@@ -234,24 +204,12 @@ namespace getfem {
     return cmk_order;
   }
 
-  void mesh::translation(base_small_vector V) {
-    for (dal::bv_visitor i(pts.index()); !i.finished(); ++i)
-      pts[i] += V;
-    pts.resort();
-  }
+  void mesh::translation(const base_small_vector &V)
+  { pts.translation(V); }
 
-  void mesh::transformation(base_matrix M) {
-    base_small_vector w(M.nrows());
-    GMM_ASSERT1(gmm::mat_nrows(M) != 0 && gmm::mat_ncols(M) == dim(),
-		"invalid dimensions for the transformation matrix");
-    for (dal::bv_visitor i(pts.index()); !i.finished(); ++i) {
-      w = pts[i];
-      gmm::resize(pts[i], gmm::mat_nrows(M));
-      gmm::mult(M,w,pts[i]);
-    }
-    dimension = gmm::mat_nrows(M);
+  void mesh::transformation(const base_matrix &M) {
+    pts.transformation(M);
     if (Bank_info) { delete Bank_info; Bank_info = 0; }
-    pts.resort();
   }
 
   void mesh::bounding_box(base_node& Pmin, base_node& Pmax) const {
@@ -267,7 +225,6 @@ namespace getfem {
   }
 
   void mesh::clear(void) {
-    dimension = dim_type(-1);
     mesh_structure::clear();
     pts.clear();
     gtab.clear(); trans_exists.clear();
@@ -344,7 +301,7 @@ namespace getfem {
 						   size_type n) const {
     bgeot::pgeometric_trans pgt = trans_of_convex(ic);
     bgeot::pgeotrans_precomp pgp
-      = bgeot::geotrans_precomp(pgt, &(pgt->geometric_nodes()));
+      = bgeot::geotrans_precomp(pgt, &(pgt->geometric_nodes()), 0);
     base_matrix G(dim(),pgt->nb_points());
     vectors_to_base_matrix(G,points_of_convex(ic));
     bgeot::geotrans_interpolation_context
@@ -365,7 +322,7 @@ namespace getfem {
 							 size_type n) const {
     bgeot::pgeometric_trans pgt = trans_of_convex(ic);
     bgeot::pgeotrans_precomp pgp
-      = bgeot::geotrans_precomp(pgt, &pgt->geometric_nodes());
+      = bgeot::geotrans_precomp(pgt, &pgt->geometric_nodes(), 0);
     base_matrix G(dim(),pgt->nb_points());
     vectors_to_base_matrix(G,points_of_convex(ic));
     bgeot::geotrans_interpolation_context
@@ -376,9 +333,8 @@ namespace getfem {
   scalar_type  mesh::convex_area_estimate(size_type ic, size_type deg) const { 
     base_matrix G;
     bgeot::vectors_to_base_matrix(G, points_of_convex(ic));
-    papprox_integration pai = 
-      classical_approx_im(trans_of_convex(ic), deg)->approx_method();
-    return getfem::convex_area_estimate(trans_of_convex(ic), G, pai);
+    return getfem::convex_area_estimate
+      (trans_of_convex(ic), G, classical_approx_im(trans_of_convex(ic), deg));
   }
 
   scalar_type  mesh::convex_quality_estimate(size_type ic) const { 
@@ -451,8 +407,6 @@ namespace getfem {
 	                       || tmp[0] == '.')
 	  { tmpv[d++] = atof(tmp.c_str()); bgeot::get_token(ist, tmp); }
 	please_get = false;
-	if (dimension == dim_type(-1)) dimension = d;
-	else GMM_ASSERT1(dimension == d, "Points of different dimensions.");
 	base_node v(d);
 	for (size_type i = 0; i < d; i++) v[i] = tmpv[i];
 	size_type ipl = add_point(v);
@@ -662,7 +616,7 @@ namespace getfem {
       vectors_to_base_matrix
 	(G, bgeot::equilateral_simplex_of_reference(N)->points());
       gmm::mult(G, bgeot::geotrans_precomp
-		(pgt, &pgt->convex_ref()->points())->grad(0), Gr);
+		(pgt, &pgt->convex_ref()->points(), 0)->grad(0), Gr);
       gmm::lu_inverse(Gr);
       pbm[N-1].swap(Gr);
     }
@@ -672,16 +626,17 @@ namespace getfem {
   
   scalar_type convex_area_estimate(bgeot::pgeometric_trans pgt,
 				   const base_matrix& G, 
-				   const approx_integration *pai) {
+				   pintegration_method pi) {
     double area(0);
     static bgeot::pgeometric_trans pgt_old = 0;
-    static papprox_integration pai_old = 0;
     static bgeot::pgeotrans_precomp pgp = 0;
-    static pintegration_method pim = 0;
-    if (pgt_old != pgt || pai_old != pai) {
+    static pintegration_method pim_old = 0;
+    papprox_integration pai = get_approx_im_or_fail(pi);
+    if (pgt_old != pgt || pim_old != pi) {
       pgt_old = pgt;
+      pim_old = pi;
       pgp = bgeot::geotrans_precomp
-	(pgt,&pai->integration_points());
+	(pgt,&pai->integration_points(), pi);
     }
     bgeot::geotrans_interpolation_context gic(pgp, 0, G);
     for (size_type i = 0; i < pai->nb_points_on_convex(); ++i) {
@@ -702,7 +657,7 @@ namespace getfem {
     static bgeot::pgeotrans_precomp pgp = 0;
     if (pgt_old != pgt) {
       pgt_old=pgt;
-      pgp=bgeot::geotrans_precomp(pgt, &pgt->convex_ref()->points());
+      pgp=bgeot::geotrans_precomp(pgt, &pgt->convex_ref()->points(), 0);
     }
 
     size_type n = (pgt->is_linear()) ? 1 : pgt->nb_points();
@@ -727,7 +682,7 @@ namespace getfem {
     static bgeot::pgeotrans_precomp pgp = 0;
     if (pgt_old != pgt) {
       pgt_old=pgt;
-      pgp=bgeot::geotrans_precomp(pgt, &pgt->convex_ref()->points());
+      pgp=bgeot::geotrans_precomp(pgt, &pgt->convex_ref()->points(), 0);
     }
     size_type N = G.nrows();
     size_type n = (pgt->is_linear()) ? 1 : pgt->nb_points();
@@ -916,8 +871,9 @@ namespace getfem {
 	mesh2.add_convex(pgt, &ipt[0]);
       }
 
+      // if (pspt) dal::del_stored_object(pspt);
       pspt = bgeot::store_point_tab(mesh2.points());
-      pgp = bgeot::geotrans_precomp(pgt, pspt);
+      pgp = bgeot::geotrans_precomp(pgt, pspt, 0);
     }
     
     base_node pt(n);
@@ -949,7 +905,7 @@ namespace getfem {
 	if (ipp == i1) ++found;
 	if (ipp == i2) ++found;
       }
-      assert(found <= 2);
+      GMM_ASSERT1(found <= 2, "Invalid convex with repeated nodes ");
       if (found == 2) ipt.push_back(cv);
     }
   }
@@ -1008,6 +964,7 @@ namespace getfem {
 
   void mesh::Bank_build_green_simplexes(size_type ic,
 					std::vector<size_type> &ipt) {
+    GMM_ASSERT1(convex_index().is_in(ic), "Internal error");
     size_type igs = Bank_info->green_simplices.add(green_simplex());
     green_simplex &gs(Bank_info->green_simplices[igs]);
     std::vector<base_node> pt_tab(nb_points_of_convex(ic));
@@ -1020,7 +977,8 @@ namespace getfem {
     size_type d = ipt.size() - 1, n = structure_of_convex(ic)->dim();
     static size_type d0 = 0;
     static bgeot::pstored_point_tab pspt1 = 0;
-    mesh &mesh1 = dal::singleton<mesh_cache_for_Bank_build_green_simplexes>::instance();
+    mesh &mesh1
+      = dal::singleton<mesh_cache_for_Bank_build_green_simplexes>::instance();
     if (d0 != d) {
       d0 = d;
       Bank_build_first_mesh(mesh1, d);
@@ -1050,7 +1008,7 @@ namespace getfem {
     for (size_type i = 0; i < ipt.size(); ++i)
       gs.ipt_loc[i] = loc_ind[gs.ipt_loc[i]];
 
-    bgeot::pgeotrans_precomp pgp = bgeot::geotrans_precomp(pgt1, pspt1);
+    bgeot::pgeotrans_precomp pgp = bgeot::geotrans_precomp(pgt1, pspt1, 0);
   
     std::vector<size_type> ipt1(pspt1->size());
     base_node pt(n);
@@ -1082,13 +1040,14 @@ namespace getfem {
         
     
     bgeot::pstored_point_tab pspt3 = bgeot::store_point_tab(mesh3.points());
-    pgp = bgeot::geotrans_precomp(pgt, pspt3);
+    pgp = bgeot::geotrans_precomp(pgt, pspt3, 0);
     
     ipt1.resize(pspt3->size());
     for (size_type ip = 0; ip < pspt3->size(); ++ip) {
       pgp->transform(points_of_convex(ic), ip, pt);
       ipt1[ip] = add_point(pt);
     }
+    // dal::del_stored_object(pspt3);
     
     ipt2.resize(pgt->nb_points());
     for (size_type icc = 0; icc < mesh3.nb_convex(); ++icc) {
@@ -1122,7 +1081,6 @@ namespace getfem {
     std::vector<size_type> ipt;
     edge_set marked_convexes;
     while (Bank_info->edges.size()) {
-     
       marked_convexes.clear();
       b = convex_index();
       edge_set::const_iterator it = Bank_info->edges.begin();
