@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2004-2006 Julien Pommier
+// Copyright (C) 2004-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -20,10 +20,11 @@
 //
 //========================================================================
 
-#include <dal_singleton.h>
-#include <getfem_mesh_slicers.h>
-#include <getfem_mesh_slice.h>
-#include <bgeot_geotrans_inv.h>
+#include "getfem/dal_singleton.h"
+#include "getfem/getfem_mesh_slicers.h"
+#include "getfem/getfem_mesh_slice.h"
+#include "getfem/bgeot_geotrans_inv.h"
+#include "getfem/getfem_mesh_level_set.h"
 
 namespace getfem {
   const float slicer_action::EPS = 1e-13;
@@ -117,7 +118,8 @@ namespace getfem {
     bool ref_pts_changed = false;
     if (ms.cvr != ms.prev_cvr || defdata->pmf->fem_of_element(ms.cv) != pf) {
       pf = defdata->pmf->fem_of_element(ms.cv);
-      if (pf->need_G()) bgeot::vectors_to_base_matrix(G, defdata->pmf->linked_mesh().points_of_convex(ms.cv));
+      if (pf->need_G()) 
+	bgeot::vectors_to_base_matrix(G, defdata->pmf->linked_mesh().points_of_convex(ms.cv));
     }
     /* check that the points are still the same -- or recompute the fem_precomp */
     std::vector<base_node> ref_pts2; ref_pts2.reserve(ms.nodes_index.card());
@@ -131,13 +133,13 @@ namespace getfem {
       ref_pts.swap(ref_pts2);
       fprecomp.clear();
     }
-    pfem_precomp pfp = fprecomp(pf, store_point_tab(ref_pts));
-    
+    bgeot::pstored_point_tab pspt = store_point_tab(ref_pts);
+    pfem_precomp pfp = fprecomp(pf, pspt);
     defdata->copy(ms.cv, coeff);
     
     base_vector val(ms.m.dim());
     size_type cnt = 0;
-    fem_interpolation_context ctx(ms.pgt, pfp, 0, G, ms.cv);
+    fem_interpolation_context ctx(ms.pgt, pfp, 0, G, ms.cv, size_type(-1));
     for (dal::bv_visitor i(ms.nodes_index); !i.finished(); ++i, ++cnt) {
       ms.nodes[i].pt.resize(defdata->pmf->get_qdim());
       ctx.set_ii(cnt);
@@ -284,9 +286,9 @@ namespace getfem {
 
     /* signalement des points qui se trouvent pile-poil sur la bordure */
     if (pt_bin.card()) {
-      if (ms.fcnt == dim_type(-1)) DAL_THROW(dal::internal_error, 
-					  "too much {faces}/{slices faces} in the convex " << ms.cv 
-					  << " (nbfaces=" << ms.fcnt << ")");
+      GMM_ASSERT1(ms.fcnt != dim_type(-1), 
+		  "too much {faces}/{slices faces} in the convex " << ms.cv 
+		  << " (nbfaces=" << ms.fcnt << ")");
       for (dal::bv_visitor cnt(pt_bin); !cnt.finished(); ++cnt) {
 	ms.nodes[cnt].faces.set(ms.fcnt);
       }
@@ -334,7 +336,7 @@ namespace getfem {
 	  x0 = slm.points_of_face_of_convex(slmcv,f)[0];
 	  base_node A = slm.points_of_face_of_convex(slmcv,f)[1] - x0;
 	  base_node B = slm.points_of_face_of_convex(slmcv,f)[2] - x0;
-	  base_node G = dal::mean_value(slm.points_of_convex(slmcv).begin(),slm.points_of_convex(slmcv).end());
+	  base_node G = gmm::mean_value(slm.points_of_convex(slmcv).begin(),slm.points_of_convex(slmcv).end());
 	  n.resize(3);
 	  n[0] = A[1]*B[2] - A[2]*B[1];
 	  n[1] = A[2]*B[0] - A[0]*B[2];
@@ -365,7 +367,8 @@ namespace getfem {
   }
 
   /* isosurface computations */
-  void slicer_isovalues::prepare(size_type cv, const mesh_slicer::cs_nodes_ct& nodes, 
+  void slicer_isovalues::prepare(size_type cv,
+				 const mesh_slicer::cs_nodes_ct& nodes, 
 				 const dal::bit_vector& nodes_index) {
     pt_in.clear(); pt_bin.clear();
     std::vector<base_node> refpts(nodes.size());
@@ -373,6 +376,7 @@ namespace getfem {
     base_vector coeff;
     base_matrix G;
     pfem pf = mfU->pmf->fem_of_element(cv);
+    if (pf == 0) return;
     fem_precomp_pool fprecomp;
     if (pf->need_G()) 
       bgeot::vectors_to_base_matrix(G, mfU->pmf->linked_mesh().points_of_convex(cv));
@@ -382,7 +386,7 @@ namespace getfem {
     //cerr << "cv=" << cv << ", val=" << val << ", coeff=" << coeff << endl;
     base_vector v(1); 
     fem_interpolation_context ctx(mfU->pmf->linked_mesh().trans_of_convex(cv),
-				  pfp, 0, G, cv);
+				  pfp, 0, G, cv, size_type(-1));
     for (dal::bv_visitor i(nodes_index); !i.finished(); ++i) {
       v[0] = 0;
       ctx.set_ii(i);
@@ -502,7 +506,7 @@ namespace getfem {
 	assert(m.points_index().is_in(pid.at(ms.simplexes.at(i).inodes[j])));
       }
       m.add_convex(bgeot::simplex_geotrans(ms.simplexes[i].dim(),1),
-		   dal::index_ref_iterator(pid.begin(),
+		   gmm::index_ref_iterator(pid.begin(),
 					   ms.simplexes[i].inodes.begin()));
     }
   }
@@ -512,10 +516,10 @@ namespace getfem {
 
     base_node G;
     if (ms.face < dim_type(-1))
-      G = dal::mean_value(ms.m.points_of_face_of_convex(ms.cv, ms.face).begin(), 
+      G = gmm::mean_value(ms.m.points_of_face_of_convex(ms.cv, ms.face).begin(), 
 			  ms.m.points_of_face_of_convex(ms.cv, ms.face).end());
     else
-      G = dal::mean_value(ms.m.points_of_convex(ms.cv).begin(), 
+      G = gmm::mean_value(ms.m.points_of_convex(ms.cv).begin(), 
 			  ms.m.points_of_convex(ms.cv).end());    
     for (dal::bv_visitor i(ms.nodes_index); !i.finished(); ++i)
       ms.nodes[i].pt = G + coef*(ms.nodes[i].pt - G);
@@ -544,6 +548,16 @@ namespace getfem {
 
   /* -------------------- member functions of mesh_slicer -------------- */
 
+  mesh_slicer::mesh_slicer(const mesh_level_set &mls_) :
+    m(mls_.linked_mesh()), mls(&mls_), pgt(0), cvr(0) {}
+  mesh_slicer::mesh_slicer(const mesh& m_) : 
+    m(m_), mls(0), pgt(0), cvr(0) {}
+
+  void mesh_slicer::using_mesh_level_set(const mesh_level_set &mls_) { 
+    mls = &mls_;
+    GMM_ASSERT1(&m == &mls->linked_mesh(), "different meshes");
+  }
+
   void mesh_slicer::pack() {
     std::vector<size_type> new_id(nodes.size());
     size_type ncnt = 0;
@@ -580,7 +594,9 @@ namespace getfem {
   static void flag_points_on_faces(const bgeot::pconvex_ref& cvr, 
                                    const std::vector<base_node>& pts, 
                                    std::vector<slice_node::faces_ct>& faces) {
-    if (cvr->structure()->nb_faces() > 32) DAL_THROW(std::out_of_range, "won't work for convexes with more than 32 faces (hardcoded limit)");
+    GMM_ASSERT1(cvr->structure()->nb_faces() <= 32,
+		"won't work for convexes with more than 32 faces "
+		"(hardcoded limit)");
     faces.resize(pts.size());
     for (size_type i=0; i < pts.size(); ++i) {
       faces[i].reset();      
@@ -597,6 +613,7 @@ namespace getfem {
     cv_dim = cvr->structure()->dim();
     cv_nbfaces = cvr->structure()->nb_faces();
     fcnt = cvr->structure()->nb_faces();
+    discont = (mls && mls->is_convex_cut(cv));
   }
 
   void mesh_slicer::apply_slicers() {
@@ -616,7 +633,7 @@ namespace getfem {
       base_matrix M(N,N);
       for (size_type i=1; i < N+1; ++i) {
 	base_small_vector d = nodes[s.inodes[i]].pt - nodes[s.inodes[0]].pt;
-	dal::copy_n(d.const_begin(), N, M.begin() + (i-1)*N);
+	gmm::copy_n(d.const_begin(), N, M.begin() + (i-1)*N);
       }
       scalar_type J = lu_det(M);
       //cout << " lu_det = " << J << "\n";	
@@ -635,6 +652,24 @@ namespace getfem {
     exec_(&nrefine[0], 1, cvlst);
   }
 
+  static bool check_orient(size_type cv, bgeot::pgeometric_trans pgt, const mesh& m) {
+    if (pgt->dim() == m.dim() && m.dim()>=2) { /* no orient check for 
+						  convexes of lower dim */
+      base_matrix G; bgeot::vectors_to_base_matrix(G,m.points_of_convex(cv));
+      base_node g(pgt->dim()); g.fill(.5); 
+      base_matrix pc; pgt->poly_vector_grad(g,pc);
+      base_matrix K(pgt->dim(),pgt->dim());
+      gmm::mult(G,pc,K);
+      scalar_type J = gmm::lu_det(K);
+      // bgeot::geotrans_interpolation_context ctx(pgp,0,G);
+      // scalar_type J = gmm::lu_det(ctx.B()); // pb car inverse K mme
+      if (J < 0) return true;
+      //cout << "cv = " << cv << ", J = " << J << "\n";
+    }
+    return false;
+  }
+
+#if OLD_MESH_SLICER
   void mesh_slicer::exec_(const short_type *pnrefine, int nref_stride, const mesh_region& cvlst) {
     std::vector<base_node> cvm_pts;
     const bgeot::basic_mesh *cvm = 0;
@@ -646,9 +681,10 @@ namespace getfem {
     cvlst.from_mesh(m);
     size_type prev_nrefine = 0;
     for (mr_visitor it(cvlst); !it.finished(); ++it) {
-      bool revert_orientation = false;
       size_type nrefine = pnrefine[it.cv()*nref_stride];
       update_cv_data(it.cv(),it.f());      
+      bool revert_orientation = check_orient(cv, pgt,m);
+
       /* update structure-dependent data */
       if (prev_cvr != cvr || nrefine != prev_nrefine) {
 	cvm = bgeot::refined_simplex_mesh_for_convex(cvr, nrefine);
@@ -663,25 +699,12 @@ namespace getfem {
       else
 	cvms = cvm; 
 
-      if (pgt->dim() == m.dim() && m.dim()>=2) { /* no orient check for convexes of lower dim */
-	base_matrix G; bgeot::vectors_to_base_matrix(G,m.points_of_convex(cv));
-	base_node g(pgt->dim()); g.fill(.5); 
-	base_matrix pc; pgt->poly_vector_grad(g,pc);
-	base_matrix K(pgt->dim(),pgt->dim());
-	gmm::mult(G,pc,K);
-	scalar_type J = gmm::lu_det(K);
-	// bgeot::geotrans_interpolation_context ctx(pgp,0,G);
-	// scalar_type J = gmm::lu_det(ctx.B()); // pb car inverse K mme
-	if (J < 0) revert_orientation = true;
-	//cout << "cv = " << cv << ", J = " << J << "\n";
-      }
-
-
       /* apply the initial geometric transformation */
       std::vector<size_type> ptsid(cvm_pts.size()); std::fill(ptsid.begin(), ptsid.end(), size_type(-1));
       simplexes.resize(cvms->nb_convex());
       nodes.resize(0);
-      for (size_type snum = 0; snum < cvms->nb_convex(); ++snum) { /* cvms should not contain holes in its convex index.. */
+      for (size_type snum = 0; snum < cvms->nb_convex(); ++snum) { 
+	/* cvms should not contain holes in its convex index.. */
 	simplexes[snum].inodes.resize(cvms->nb_points_of_convex(snum));
 	std::copy(cvms->ind_points_of_convex(snum).begin(),
 		  cvms->ind_points_of_convex(snum).end(), simplexes[snum].inodes.begin());
@@ -710,6 +733,179 @@ namespace getfem {
       apply_slicers();
     }
   }
+#endif // OLD_MESH_SLICER
+
+  template <typename CONT> 
+  static void add_degree1_convex(bgeot::pgeometric_trans pgt, const CONT &pts, 
+				 mesh &m) {
+    unsigned N = pgt->dim();
+    std::vector<base_node> v; v.reserve(N+1);
+    for (unsigned i=0; i < pgt->nb_points(); ++i) {
+      const base_node P = pgt->convex_ref()->points()[i];
+      scalar_type s = 0; 
+      for (unsigned j=0; j < N; ++j) { 
+	s += P[j]; if (P[j] == 1) { v.push_back(pts[i]); break; }
+      }
+      if (s == 0) v.push_back(pts[i]);
+    }
+    assert(v.size() == N+1);
+    base_node G = gmm::mean_value(v);
+    /*for (unsigned i=0; i < v.size();++i) 
+      v[i] = v[i] + 0.1 * (G - v[i]);*/
+    m.add_convex_by_points(bgeot::simplex_geotrans(N,1), v.begin());
+  }
+
+  const mesh& mesh_slicer::refined_simplex_mesh_for_convex_cut_by_level_set
+  (const mesh &cvm, unsigned nrefine) {
+    mesh mm; mm.copy_from(cvm);
+    while (nrefine > 1) { 
+      mm.Bank_refine(mm.convex_index());
+      nrefine /= 2;
+    }
+
+    std::vector<size_type> idx;
+    tmp_mesh.clear();
+    //cerr << "nb cv = " << tmp_mesh.convex_index().card() << "\n";
+    for (dal::bv_visitor_c ic(mm.convex_index()); !ic.finished(); ++ic) {
+      add_degree1_convex(mm.trans_of_convex(ic), mm.points_of_convex(ic), tmp_mesh);
+    }
+    /*tmp_mesh.write_to_file(std::cerr);
+      cerr << "\n";*/
+    return tmp_mesh;
+  }
+  
+  const bgeot::mesh_structure &
+  mesh_slicer::refined_simplex_mesh_for_convex_faces_cut_by_level_set
+  (size_type f) {
+    mesh &cvm = tmp_mesh;
+    tmp_mesh_struct.clear();
+    unsigned N = cvm.dim();
+    
+    dal::bit_vector pt_in_face; pt_in_face.sup(0, cvm.points().index().last_true()+1);
+    for (dal::bv_visitor ip(cvm.points().index()); !ip.finished(); ++ip)
+      if (cvr->is_in_face(f, cvm.points()[ip])) pt_in_face.add(ip);
+
+    for (dal::bv_visitor_c ic(cvm.convex_index()); !ic.finished(); ++ic) {
+      for (unsigned ff=0; ff < cvm.nb_faces_of_convex(ic); ++ff) {
+	bool face_ok = true;
+	for (unsigned i=0; i < N; ++i) {
+	  if (!pt_in_face.is_in(cvm.ind_points_of_face_of_convex(ic,ff)[i])) {
+	    face_ok = false; break;
+	  }
+	}
+	if (face_ok) {
+	  tmp_mesh_struct.add_convex(bgeot::simplex_structure(N-1), 
+				     cvm.ind_points_of_face_of_convex(ic, ff).begin());
+	}
+      }
+    }
+    return tmp_mesh_struct;
+  }
+
+  void mesh_slicer::exec_(const short_type *pnrefine, 
+			  int nref_stride, 
+			  const mesh_region& cvlst) {
+    std::vector<base_node> cvm_pts;
+    const bgeot::basic_mesh *cvm = 0;
+    const bgeot::mesh_structure *cvms = 0;
+    bgeot::geotrans_precomp_pool gppool;
+    bgeot::pgeotrans_precomp pgp = 0;
+    std::vector<slice_node::faces_ct> points_on_faces;
+    bool prev_discont = true;
+
+    cvlst.from_mesh(m);
+    size_type prev_nrefine = 0;
+    size_type prev_cv = size_type(-1);
+    for (mr_visitor it(cvlst); !it.finished(); ++it) {
+      size_type nrefine = pnrefine[it.cv()*nref_stride];
+      update_cv_data(it.cv(),it.f());
+      bool revert_orientation = check_orient(cv, pgt,m);
+
+      /* update structure-dependent data */
+      
+      /* TODO : fix levelset handling when slicing faces .. */
+      if (prev_cvr != cvr || nrefine != prev_nrefine || discont || prev_discont) {
+	if (discont) {
+	  //if (prev_cv != it.cv())
+	  cvm = &refined_simplex_mesh_for_convex_cut_by_level_set(mls->mesh_of_convex(cv), 
+								  nrefine);
+	} else {
+	  cvm = bgeot::refined_simplex_mesh_for_convex(cvr, nrefine);
+	}
+	cvm_pts.resize(cvm->points().card());
+	std::copy(cvm->points().begin(), cvm->points().end(), cvm_pts.begin());
+	pgp = gppool(pgt, store_point_tab(cvm_pts));
+	flag_points_on_faces(cvr, cvm_pts, points_on_faces);
+	prev_nrefine = nrefine;
+      }
+      if (face < dim_type(-1)) {
+	if (!discont) {
+	  cvms = bgeot::refined_simplex_mesh_for_convex_faces(cvr, nrefine)[face];
+	} else {
+	  cvms = &refined_simplex_mesh_for_convex_faces_cut_by_level_set(face);
+	}
+      } else {
+	cvms = cvm; 
+      }
+
+
+
+      /* apply the initial geometric transformation */
+      std::vector<size_type> ptsid(cvm_pts.size()); std::fill(ptsid.begin(), ptsid.end(), size_type(-1));
+      simplexes.resize(cvms->nb_convex());
+      nodes.resize(0);
+
+      base_node G;
+      for (size_type snum = 0; snum < cvms->nb_convex(); ++snum) { 
+	/* cvms should not contain holes in its convex index.. */
+	simplexes[snum].inodes.resize(cvms->nb_points_of_convex(snum));
+	std::copy(cvms->ind_points_of_convex(snum).begin(),
+		  cvms->ind_points_of_convex(snum).end(), simplexes[snum].inodes.begin());
+	if (revert_orientation) std::swap(simplexes[snum].inodes[0],simplexes[snum].inodes[1]);
+	/* store indices of points which are really used , and renumbers them */
+	if (discont) {
+	  G.resize(m.dim()); G.fill(0.);
+	  for (std::vector<size_type>::iterator itp = 
+		 simplexes[snum].inodes.begin();
+	       itp != simplexes[snum].inodes.end(); ++itp) {
+	    G += cvm_pts[*itp];
+	  }
+	  G /= simplexes[snum].inodes.size();
+	}
+	  
+
+	for (std::vector<size_type>::iterator itp = 
+	       simplexes[snum].inodes.begin();
+	     itp != simplexes[snum].inodes.end(); ++itp) {
+	  if (discont || ptsid[*itp] == size_type(-1)) {
+	    ptsid[*itp] = nodes.size();
+	    nodes.push_back(slice_node());
+	    if (!discont) {
+	      nodes.back().pt_ref = cvm_pts[*itp];
+	    } else {
+	      /* displace the ref point such that one will not interpolate
+		 on the discontinuity (yes this is quite ugly and not 
+		 robust) 
+	      */
+	      nodes.back().pt_ref = cvm_pts[*itp] + 0.01*(G - cvm_pts[*itp]);
+	    }
+	    nodes.back().faces = points_on_faces[*itp];
+	    nodes.back().pt.resize(m.dim()); nodes.back().pt.fill(0.);
+	    pgp->transform(m.points_of_convex(cv), *itp, nodes.back().pt);
+	    //nodes.back().pt = pgt->transform(G, m.points_of_convex(cv));
+	    //cerr << "G = " << G << " -> pt = " << nodes.back().pt << "\n";
+	  }
+	  *itp = ptsid[*itp];
+	}
+      }
+      //cerr << "cv = " << cv << ", cvm.nb_points_ = "<< cvm->points().size() << ", nbnodes = " << nodes.size() << ", nb_simpl=" << simplexes.size() << "\n";
+
+      apply_slicers();
+      prev_cv = it.cv();
+      prev_discont = discont;
+    }
+  }
+
 
   void mesh_slicer::exec(size_type nrefine) {
     exec(nrefine,mesh_region(m.convex_index()));
@@ -717,7 +913,7 @@ namespace getfem {
   
   /* apply slice ops to an already stored slice object */
   void mesh_slicer::exec(const stored_mesh_slice& sl) {
-    if (&sl.linked_mesh() != &m) DAL_THROW(dal::failure_error, "wrong mesh");
+    GMM_ASSERT1(&sl.linked_mesh() == &m, "wrong mesh");
     for (stored_mesh_slice::cvlst_ct::const_iterator it = sl.cvlst.begin(); it != sl.cvlst.end(); ++it) {
       update_cv_data((*it).cv_num);
       nodes = (*it).nodes;
